#ifndef MODULE_HANDLE_H
#define MODULE_HANDLE_H

#include "Own.h"
#include "Text.h"
#include "Command.h"

namespace MM
{
class Module;

class Module_handle
{
    friend class Module;

    Module_handle (Module & module, Command * adoptee)
        : module_(module), command_(adoptee) 
    {
        //command_.adopt (new Qt_command (adoptee)); //fix Interface::wrap_command()
    }

    Module          &module_;
    Own <Command>   command_;
    Text            menu_;
    Text            menu_item_;
    Text            accelerator_;
    Text            icon_;
    Text            tool_tip_;

public:
    ~Module_handle ();

    Text const &        module_name () const;
    Own <Command> &     command     ()        {return command_;}
    char const *        menu        () const  {return menu_       .c_str();}
    char const *        menu_item   () const  {return menu_item_  .c_str();}
    char const *        accelerator () const  {return accelerator_.c_str();}
    char const *        icon        () const  {return icon_       .c_str();}
    char const *        tool_tip    () const  {return tool_tip_   .c_str();}

    bool        has_menu        () const  {return menu_       .size() != 0;}
    //bool        has_menu_item   () const  {return menu_item_  .size() != 0;}
    bool        has_accelerator () const  {return accelerator_.size() != 0;}
    bool        has_icon        () const  {return icon_       .size() != 0;}
    bool        has_tool_tip    () const  {return tool_tip_   .size() != 0;}

    Module_handle & set_menu (char const * menu, char const * item)
                        {menu_ = menu;   menu_item_ = item;  return *this;}

    Module_handle & set_accelerator (char const * shortcut)
                        {accelerator_ = shortcut; return *this;}

    Module_handle & set_icon (char const * file) 
                        {icon_ = file; return *this;}

    Module_handle & set_tool_tip (char const * tool_tip)
                        {tool_tip_ = tool_tip; return *this;}
};

}//MM

#endif //MODULE_HANDLE_H
