#include "Rough_hydrophobic_interactions.h"

#include <math.h>

namespace MM
{

Rough_hydrophobic_interactions::
Rough_hydrophobic_interactions (Atom_group & atom_group, Force_field & ff)
:
    Conventional_interactions (atom_group, ff)
{
}

Rough_hydrophobic_interactions::
Rough_hydrophobic_interactions (Atom_group & atom_group,
    Bond_group & bond_group, Force_field & ff)
:
    Conventional_interactions (atom_group, bond_group, ff)
{
}

void Rough_hydrophobic_interactions::
nonbonded_potential (Atom_cache & atom)
{
    //const double l      = 0.17;//0.17 -0.774
    //const double l      = 0.06;//0.06 -0.864
    //const double l      = 0.015;//0.015 -0.878
    //const double l      = 0.005;//0.005 -0.826
    //const double l      = 1.25;//0.005 -0.826
    //const double l      = 0.11;//
    const double l      = 0.035;//

    //const double k      = 1. * (fabs (atom.charge___) - l);
    //const double k      = 1. * (pow(atom.charge___, 2) - l);
    //const double k      = 1. * (pow(atom.charge___, 4) - l);
    //const double k      = 1. * (pow(atom.charge___, 8) - l);
    //const double k      = 1. * (exp(fabs(atom.charge___)) - l);
    //const double k      = 1. * ((pow(atom.charge___, 2)/(pow(atom.charge___, 2)+0.6)) - l);
    const double k      = 1. * ((pow(atom.charge___, 4)/(pow(atom.charge___, 4)+0.50)) - l);

    const double Rm     = 10.;
    const double lIRm   = 1. / Rm;
    const double lI2Rm_3= 1. / (2. * Rm*Rm*Rm);

    double  lIepsilon = 1. / epsilon_;//fix
    double  A = 0., B = 0.;

    int     atom_number = atom.number___;
    int     count       = atom_count();

    double  charge_332  = atom.charge___ * EPS0; //331.8;
    //double  e           = 0;
    double  P_VdW = 0., P_el = 0., P_hph = 0., Yakub_Ronchi = 0.;
    double  x, y, z;
    double  r_2, lIr, lIr_6, lIr_eps;

    Force_field::VdWaals * vdw = force_field_.VdW_par [atom.mm_type___];

    double  ax = atom.x___;
    double  ay = atom.y___;
    double  az = atom.z___;

    miss_neighbours (atom);

    // 0.00315
    for (int i=atom_number+1;  i<count;  ++i)
    //for (int i=atom_number+1;  i<count;  i+=4)
    {
        Atom_cache & current_atom  = atom_[i];

        Force_field::VdWaals & tmp = vdw [current_atom.mm_type___];  //!!!
        A = tmp.A;
        B = tmp.B; 

        x = ax - current_atom.x___;
        y = ay - current_atom.y___;
        z = az - current_atom.z___;

        r_2     = x*x + y*y + z*z;
        lIr     = 1.0 / sqrt (r_2);
        lIr_eps = lIr * lIepsilon;
        lIr_6   = lIr * lIr;    lIr_6 *= lIr_6 * lIr_6;

        //Yakub_Ronchi = 1. + (r_2 * r_2 * lIr) * lI2Rm_3;
        double rIRm = r_2 * lIr * lIRm;
        Yakub_Ronchi   = 1. + .5 * rIRm * (rIRm * rIRm - 3.); 

        P_VdW += current_atom.transparency___ * 
            (A * lIr_6 - B ) * lIr_6;

        P_el += current_atom.transparency___ *
            current_atom.charge___ * charge_332 * lIr_eps * Yakub_Ronchi;

        P_hph += k * lIr * Yakub_Ronchi;
        //P_hph += (1. * (fabs (current_atom.charge___) - l)) * lIr * Yakub_Ronchi;
        //P_hph += (1. * (pow(current_atom.charge___, 2) - l)) * lIr * Yakub_Ronchi;
        //P_hph += (1. * (pow(current_atom.charge___, 4) - l)) * lIr * Yakub_Ronchi;
        //P_hph += (1. * (pow(current_atom.charge___, 8) - l)) * lIr * Yakub_Ronchi;
        //P_hph += (1. * (exp(fabs(current_atom.charge___)) - l)) * lIr * Yakub_Ronchi;
        //P_hph += (1. * ((pow(current_atom.charge___, 2)/(pow(current_atom.charge___, 2)+0.6)) - l)) * lIr * Yakub_Ronchi;
        P_hph += (1. * ((pow(current_atom.charge___, 4)/(pow(current_atom.charge___, 4)+0.50)) - l)) * lIr * Yakub_Ronchi;
    }

    process_14 (atom, false);
    restore_neighbours ();

    Van_der_Waals_potential_ += P_VdW;
    electrostatic_potential_ += P_el;
    hydrophobic_potential_   += P_hph;

    last_potential_ += P_el + P_VdW + P_hph;
}

}//MM


