#include "Model_kit.h"

#include "Cleaner.h"
#include "User.h"
#include "Atom_kit.h"
#include "Model_view_composite.h"
#include "Model.h"
#include "Rigid_body_impl.h"
#include "Model_as_md_subject.h"
#include "No_interaction.h"
#include "Box_impl.h"
#include "mm.Core/Atom_groups/Molecular_sequence_impl.h"
#include "Create.h"

namespace MM
{

Model_kit::
Model_kit (Model &model)
:
	file_name_				("New.mlm"),
    model_                  (model),
    file_                   (model),
    interaction_            (new No_interaction),
    boundary_conditions_    (model),
    rigid_body_             (new Rigid_body_impl),
    lock_count_             (0)
{
    view_.adopt (new Model_view_composite());
}

Model_kit::
Model_kit (Prototype &, Model & model)
:
    model_(model),
    file_                   (model),
    boundary_conditions_(model)//,
    //crystal_builder_(model)
{
}

void Model_kit::set_alias         (Text const & v)  {alias_       = v;}
void Model_kit::set_short_alias   (Text const & v)  {short_alias_ = v;}
//void Model_kit::set_model_name    (Text const & v)  {model_name_  = v;}
void Model_kit::set_comment_line  (Text const & v)  {comment_line_= v;}

void Model_kit::
strip ()
{
    Cleaner cleaner (model_);
    const Text zero_type = "0";

    for (int i=0;  i<model_.atom_count();  ++i)
    {
        Atom & atom = model_.atom (i);

        if (atom.kit().has_mm_type() 
            && zero_type == atom.kit().mm_type_c_str())
            cleaner.add (atom);
    }

    if (cleaner.is_empty())
        to_user().info ("There is nothing to strip.");
}

void Model_kit::
set_force_field (Text const & v)
{
    force_field_ = v;

    int count = model().atom_count();

    for (int i=0;  i<count;  ++i)
    {
        Atom     & atom = model().atom(i);
        Atom_kit & kit  = atom.kit();

        if (kit.has_fine_type())
        {
            double charge = 12345;
            char const * type = Fine_type::to 
                (kit.fine_type_name(), force_field_.c_str(), &charge);
            kit.set_mm_type (type);
            
            if (charge != 12345)
                atom.set_charge (charge);
        }
    }

    event().force_field_changed ();
}


void Model_kit::
add_view (Model_view *adoptee)
{
    //REQUIRE ("",  !has_view());
    //view_.adopt (adoptee);
    //ENSURE  ("",  has_view());

    //dynamic_cast<Model_view_composite&>(view_()).adopt (adoptee);
//    set_ownership (*adoptee, *this);
    view_().adopt (adoptee);
}

/*void Model_kit::
remove_view()
{
    REQUIRE ("",  has_view());
    delete view_.orphan();
    ENSURE  ("",  !has_view());
}//*/

Crystal_builder & Model_kit::
crystal_builder ()
{
    if (crystal_builder_.is_not_valid())
        crystal_builder_.adopt (prototype <Crystal_builder> (). clone(model()));
    return crystal_builder_();
}

Sequence_editor & Model_kit::
sequence_editor ()
{
    if (sequence_editor_.is_not_valid())
        sequence_editor_.adopt (new Sequence_editor (model()));
    return sequence_editor_();
}

Add_builder & Model_kit::
add_builder ()
{
    if (add_builder_.is_not_valid())
        add_builder_.adopt (prototype <Add_builder>().clone (model()));
    return add_builder_();
}

Box & Model_kit::
crystal_cell ()
{
    if (!has_crystal_cell ())
        crystal_cell_.adopt (new Box_impl());
    return crystal_cell_();
}

Box const & Model_kit::
crystal_cell () const
{
    if (!has_crystal_cell ())
        crystal_cell_.adopt (new Box_impl());
    return crystal_cell_();
}

void Model_kit::
erase (Model_view &adoptee)
{
    delete view_().orphan (&adoptee);
    if (view_().count() == 0)
        model().erase();
}

MD_subject * Model_kit::
create_md_interface ()
{
    //return new Model_as_md_subject (model_, interaction());
    return new Model_as_md_subject (model_);
}


Arbitrary_atom_group & Model_kit::
ordered_atoms ()
{
    if(!has_ordered_atoms())
        ordered_atoms_.adopt (new Arbitrary_atom_group);
    return ordered_atoms_();
}

}//MM
