#include "Form_monitors.h"

#include "Form_main_window.h"
#include "Qt_application.h"
#include "Project.h"
#include "Project_kit.h"
#include "Protocol_stage.h"
#include "Protocol_cursor.h"

#include "Form_gnuplot_graph.h"//fix 

#include <QAbstractListModel>
#include <QHeaderView>
#include <QItemDelegate>
#include <QCheckBox>

namespace MM
{

Form_monitors::
Form_monitors ()
{
    setupUi (this);

    //=========================================================================

    class Monitor_Delegate : public QItemDelegate
    {
    public:
        Monitor_Delegate (QObject *parent = 0) : QItemDelegate(parent) {}

        QWidget * createEditor (QWidget *                    parent, 
                                const QStyleOptionViewItem & option,
                                const QModelIndex &          index) const
        {
            if (index.column () == 1)
            {
                QCheckBox * editor = new QCheckBox (parent);
                editor->installEventFilter
                    (const_cast <Monitor_Delegate*> (this));
                return editor;
            }
            else
                return QItemDelegate::createEditor (parent, option, index);
        }

        void setEditorData (QWidget *editor, const QModelIndex &index) const
        {
            if (index.column () == 1)
            {
                bool on = index.model()->data(index, Qt::DisplayRole).toBool();
                QCheckBox * checkBox = static_cast <QCheckBox*> (editor);
                checkBox->setCheckState (on ? Qt::Checked : Qt::Unchecked);
            }
            else
                QItemDelegate::setEditorData (editor, index);
        }
        
        void setModelData (QWidget *            editor, 
                           QAbstractItemModel * model,
                           const QModelIndex &  index) const
        {
            if (index.column () == 1)
            {
                QCheckBox * checkBox = static_cast <QCheckBox*> (editor);
                bool on = checkBox->checkState () == Qt::Checked;
                model->setData (index, on);
            }
            else
                QItemDelegate::setModelData (editor, model, index);
        }

        void updateEditorGeometry (QWidget *                    editor,
                                   const QStyleOptionViewItem & option, 
                                   const QModelIndex &          index) const
        {
            if (index.column () == 1)
            {
                editor->setGeometry (option.rect);
            }
            else
                QItemDelegate::updateEditorGeometry (editor, option, index);
        }
    };

    //=========================================================================

    class Monitor_model : public QAbstractListModel
    {
    public:
        int rowCount (const QModelIndex &) const  
        {
            Project_kit & kit = Project::singleton().kit();
            int project_monitor_count = kit.monitor_count();
            int stage_monitor_count   = kit.protocol().cursor().is_valid() 
                ? kit.protocol().cursor().current().monitor_count()
                : 0;

            return project_monitor_count + stage_monitor_count;
        }

        int columnCount (const QModelIndex &) const  {return 2;}

        QVariant data (const QModelIndex & index, int role) const
        {
            if (index.isValid())
            {                 
                Project &        project  = Project::singleton();
                Protocol &       protocol = project.kit().protocol();
                Protocol_stage & stage    = protocol.cursor().current();
                int project_monitor_count = project.kit().monitor_count();

                Monitor *monitor =0;
                if (index.row() < project_monitor_count)
                    monitor = &Project::singleton().kit().monitor(index.row());
                else
                    monitor = &stage.monitor
                        (index.row() - project_monitor_count);
            
                if(role == Qt::DisplayRole)
                {
                    switch (index.column())
                    {
                    case 0: return QString (monitor->title().c_str());
                    case 1: return QVariant();
                    default: FLAW ("No such column.");
                    }
                }
                else if(role == Qt::CheckStateRole && index.column() == 1)
                {
                    return monitor->is_on ();
                }
                /*else if(role == Qt::FontRole && index.column() == 0)
                {
                    if (!monitor->is_on ())
                    {
                        QFont font;
                        font.setWeight (0);
                        return font;
                    }
                }//*/
                else if(role == Qt::ForegroundRole && index.column() == 0)
                {
                    if (!monitor->is_on ())
                    {
                        return QColor (170, 170, 170);
                    }
                }
            }

            return QVariant();
        }

        QVariant headerData (int section, 
                             Qt::Orientation orientation, 
                             int role) const
        {
            if (role != Qt::DisplayRole)    return QVariant();
                
            if (orientation == Qt::Horizontal)
            {
                switch (section)
                {
                case 0: return QString ("Monitor");
                case 1: return QString ("on/off");
                default: FLAW ("No such column.");
                }
            }
            else
                return QString("Row %1").arg(section);
        }
 
        Qt::ItemFlags flags (const QModelIndex &index) const
        {
            if (!index.isValid())
                return Qt::ItemIsEnabled;

            if (index.column() == 1)
                return QAbstractItemModel::flags(index) | Qt::ItemIsEditable;

            return QAbstractItemModel::flags(index);
        }

        bool setData(const QModelIndex &index, const QVariant &value, int role)
        {
            if (!index.isValid() || role != Qt::EditRole)
                return false;

            Project &        project  = Project::singleton();
            Protocol &       protocol = project.kit().protocol();
            Protocol_stage & stage    = protocol.cursor().current();
            int project_monitor_count = project.kit().monitor_count();

            Monitor *monitor;
            if (index.row() < project_monitor_count)
                monitor = &Project::singleton().kit().monitor(index.row());
            else
                monitor = &stage.monitor
                    (index.row() - project_monitor_count);

            switch (index.column())
            {
            case 0: return false;
            case 1: monitor->turn_on (value.toBool()); break;
                     
            default: FLAW ("Column number is out of range.");
            }

            emit dataChanged (index, index);
            emit layoutChanged ();
            return true;
        }//*/
    };

    //=========================================================================

    treeView->setItemDelegate (new Monitor_Delegate (this));
    treeView->setModel (new Monitor_model);

    //treeView->header()->setStretchLastSection (false);
    treeView->header()->setResizeMode   (QHeaderView::ResizeToContents);

    treeView->setEditTriggers (QAbstractItemView::CurrentChanged);
    //treeView->setEditTriggers (QAbstractItemView::AllEditTriggers);

    connect (treeView,  SIGNAL(doubleClicked (const QModelIndex &)),
             this,      SLOT  (show_details  (const QModelIndex &)));

    connect (treeView,  SIGNAL(clicked (const QModelIndex &)),
             this,      SLOT  (switch_details (const QModelIndex &)));
}

void Form_monitors::
pop_up ()
{
    Form_main_window &main_window 
        = Qt_application::singleton().main_window();

    if (exist())
        main_window.pop_up (singleton());
    else
        main_window.add_right_control (singleton(), "Monitors");
}

void Form_monitors::
show_details  (const QModelIndex & index)
{
    if (!index.isValid())
        return;

    Project &        project  = Project::singleton();
    Protocol &       protocol = project.kit().protocol();
    Protocol_stage & stage    = protocol.cursor().current();
    int project_monitor_count = project.kit().monitor_count();

    Monitor *monitor =0;
    if (index.row() < project_monitor_count)
        monitor = &Project::singleton().kit().monitor(index.row());
    else
        monitor = &stage.monitor
            (index.row() - project_monitor_count);

    if (monitor != 0)
        monitor->show ();
}

void Form_monitors::
switch_details  (const QModelIndex & index)
{
    if (!index.isValid() || !Form_gnuplot_graph::exist())
        return;

    show_details (index);
}

}//MM
