#include "Form_protocol.h"

#include "Form_main_window.h"
#include "Form_monitors.h"
#include "Qt_application.h"

#include "Protocol.h"
#include "Protocol_stage.h"
#include "Protocol_cursor.h"

#include "Project.h"
#include "Project_kit.h"

#include "Model.h"
#include "Model_kit.h"
#include "Model_proxy.h"

#include "Path.h"
#include "User.h"
#include "Log.h"
#include "Pause_stage.h"

#include <QDirModel>
#include <QStringList>
#include <QHeaderView>

namespace MM
{

Form_protocol::
Form_protocol ()
{
    setupUi (this);

    class Protocol_dir_model : public QAbstractListModel
    {
        QDir dir_;
    
    public:
        Protocol_dir_model ()
        :     
            dir_ (Path::protocol().c_str(),     "*.protocol", 
                  QDir::Name | QDir::IgnoreCase, QDir::Files)
        {
        }

        int rowCount (const QModelIndex &) const  {return dir_.count();}

        QVariant data (const QModelIndex & index, int role) const
        {
            if (index.isValid() && 
                role == Qt::DisplayRole)
            {
                QFileInfoList list = dir_.entryInfoList ();
                if (index.row() < list.count())
                    return list.at (index.row()).baseName ();
            }

            return QVariant();
        }
    };
    listView->setModel (new Protocol_dir_model);

    connect (listView,  SIGNAL(clicked         (const QModelIndex &)),
             this,      SLOT  (choose_protocol (const QModelIndex &)));  
    
    //=========================================================================

    class Protocol_model : public QAbstractListModel
    {
    public:
        int rowCount    (const QModelIndex &) const  
        {
            return Project::singleton().kit().protocol().stage_count();
        }

        int columnCount (const QModelIndex &) const  {return 6;}

        QVariant data (const QModelIndex & index, int role) const
        {
            if (index.isValid() && 
                role == Qt::DisplayRole)
            {
                Protocol &  protocol = Project::singleton().kit().protocol();
                Protocol_stage & stage = protocol.stage (index.row());
                
                switch (index.column())
                {
                case 0: return QString (stage.intention().title().c_str());
                case 1: return QString (stage.option("Method").c_str());
                case 2: return QString (stage.option("Length").c_str());
                case 3: return QString (stage.option("T").c_str());
                case 4: return QString (stage.option("P").c_str());
                case 5: return QString (stage.option("Comment").c_str());
                default: FLAW ("No such column.");
                }
            }

            return QVariant();
        }

        QVariant headerData (int section, 
                             Qt::Orientation orientation, 
                             int role) const
        {
            if (role != Qt::DisplayRole)    return QVariant();
                
            if (orientation == Qt::Horizontal)
            {
                switch (section)
                {
                case 0: return QString ("Intention");
                case 1: return QString ("Method");
                case 2: return QString ("Length");
                case 3: return QString ("T");
                case 4: return QString ("P");
                case 5: return QString ("Comment");
                default: FLAW ("No such column.");
                }
            }
            else
                return QString("Row %1").arg(section);
        }
 
        Qt::ItemFlags flags (const QModelIndex &index) const
        {
            if (!index.isValid())
                return Qt::ItemIsEnabled;

            if (index.column() < 2)
                return QAbstractItemModel::flags(index);

            return QAbstractItemModel::flags(index) | Qt::ItemIsEditable;
        }

        bool setData(const QModelIndex &index, const QVariant &value, int role)
        {
            if (!index.isValid() || role != Qt::EditRole)
                return false;

            Protocol &       protocol = Project::singleton().kit().protocol();
            Protocol_stage & stage = protocol.stage (index.row());
            Text             new_value = value.toString().ascii();

            switch (index.column())
            {
            case 0: return false;
            case 1: return false;
            case 2: stage.set_option ("Length", new_value);  break;
            case 3: stage.set_option ("T", new_value);       break;
            case 4: stage.set_option ("P", new_value);       break;
            case 5: stage.set_option ("Comment", new_value); break;
                     
            default: FLAW ("Column number is out of range.");
            }

            emit dataChanged (index, index);
            emit layoutChanged ();
            return true;
        }
    };
    treeView->setModel (new Protocol_model);
    treeView->header()->setStretchLastSection (true);
    treeView->header()->setResizeMode (QHeaderView::ResizeToContents);
    treeView->setSelectionMode (QAbstractItemView::SingleSelection);

    connect (treeView,  SIGNAL(clicked     (const QModelIndex &)),
             this,      SLOT  (choose_step (const QModelIndex &)));  

    //connect (treeView,  SIGNAL(doubleClicked (const QModelIndex &)),
    //         this,      SLOT  (show_options  (const QModelIndex &)));  
    //connect (treeView,  SIGNAL(pressed (const QModelIndex &)),
    //         this,      SLOT  (show_options  (const QModelIndex &)));  
    //connect (treeView,  SIGNAL(clicked (const QModelIndex &)),
    //         this,      SLOT  (show_options  (const QModelIndex &))); 
    
    connect (listView,  SIGNAL(clicked (const QModelIndex &)),
             this,      SLOT  (show_options  (const QModelIndex &)));  

    connect (pushButton_Run,    SIGNAL(clicked()),  this,   SLOT(run()));
    connect (pushButton_Stop,   SIGNAL(clicked()),  this,   SLOT(stop()));
    connect (pushButton_Tune,   SIGNAL(clicked()),  this,   SLOT(tune ()));
    connect (pushButton_Start,  SIGNAL(clicked()),  this,   SLOT(start()));

    connect (pushButton_Options,    SIGNAL(clicked()),  
             this,                  SLOT  (show_options()));
}

void Form_protocol::
choose_protocol  (const QModelIndex & index)
{
    QString file_name = index.data().toString() + ".protocol";
    Project::singleton().kit().load_protocol
         (Path::protocol() + file_name.ascii());

    //treeView->setCurrentIndex (treeView->model()->index(0,0));
    //treeView->reset ();

    //choose_step  (index);
    choose_step  (treeView->model()->index(0,0));
    
//    to_user (). status (file_name.ascii());
}

void Form_protocol::
choose_step  (const QModelIndex & index)
{
    Project::singleton().kit().protocol().cursor().set (index.row());
    //Project::singleton().kit().protocol().cursor().current().update();

    if (Form_monitors::exist())
    {
        QTreeView * tree  = Form_monitors::singleton()->treeView;
        QModelIndex index = tree->currentIndex ();
        tree->reset ();
        Form_monitors::singleton()->switch_details (index);
        //Form_monitors::singleton()->show_details (index);
    }

    treeView->reset ();
    //treeView->setCurrentIndex (treeView->model()->index (0,0));
    //treeView->setCurrentIndex (treeView->currentIndex ());
    //treeView->setCurrentIndex (index);

    //show_options  (index);
}

void Form_protocol::
//show_options  (const QModelIndex & index)
//show_options  (const QModelIndex & )
show_options  ()
{
    Protocol_cursor & cursor = Project::singleton().kit().protocol().cursor();
    if (!cursor.is_valid())
        return;

    cursor.current().edit();

    if (Form_monitors::exist())
        Form_monitors::singleton()->treeView->reset ();
}

void Form_protocol::
showEvent (QShowEvent * event)
{
    if (Project::singleton().model_count() == 0)
        reset_current_model (Model::none());
    else
        reset_current_model (Project::singleton().current_model());


    if (run_state_ == 0)
        Run_state lock (*this);
    else
        run_state_->apply (*this);

    QWidget::showEvent (event);
}

void Form_protocol::
reset_current_model (Model & model)
{
    if (&model == &Model::none())
        frame->setEnabled (false);
    else
    {
        frame->setEnabled (true);
        update ();
        if (Form_monitors::exist())
            Form_monitors::singleton()->update ();
    }

    // select current step
    Project::singleton().kit().protocol().cursor(). update ();
}

void Form_protocol::
pop_up ()
{
    Form_main_window &main_window 
        = Qt_application::singleton().main_window();

    if (exist())
        main_window.pop_up (singleton());
    else
        main_window.add_bottom_control (singleton(), "Protocol");
}

void Form_protocol::
run ()
{
    Own <Run_state> lock (new Run_state (*this));
    stop_flag_ = false;

    Protocol & protocol = Project::singleton().kit().protocol();
    protocol.set_model  (Project::singleton().current_model());
    Model_kit::Lock model_lock = 
                        Project::singleton().current_model().kit().lock();
    log() << "\nStart protocol." << "\n";

    for (Protocol_cursor & cursor = protocol.cursor();
         cursor.is_valid();     cursor.next())
    {
        Protocol_stage & stage = cursor.current ();
        
        if (!stage.intention().check())
        {
            if (!to_user().question (stage.intention().last_message () 
                                    + "\nDo you want to continue?"))
                break;
        }

        log() 
            << "\tStart step " << cursor.index() 
            << ": " << stage.intention().title()
            << " "  << stage.option ("Method")
            << " "  << stage.option ("Length")
            << " "  << stage.option ("T")
            << " "  << stage.option ("P")
            << " "  << stage.option ("Comment")
            << "\n";
        
        struct Process_events : public Command
        {
            void execute () {qApp->processEvents (QEventLoop::AllEvents, 100);}
        };
        stage.at_every_iteration (new Process_events);

        try
        {
            stage.execute ();
        }
        catch (Flaw & exception)
        {
            log () << "###\tFlaw: " << exception.message() << "\n";
            to_user().error (exception.message());
            break;
        }
        catch (...)
        {
            log () << "###\tUnknown exception!\n";
            to_user().error ("Unknown exception.");
            break;
        }

        if (!stage.intention().was_reached())
        {
            if (!to_user().question (stage.intention().last_message () 
                                    + "\nDo you want to continue?"))
                stop_flag_ = true;
        }

        if (!stop_flag_)
        {
            Pause_stage pause (protocol, "Pause_step 2sec");
            pause.at_every_iteration (new Process_events);
            pause.execute ();
            if (pause.was_stopped())
                stop_flag_ = true;
        }

        if (stop_flag_)
        {
            log () << "\t";
            log () << "User STOP.\n";
            protocol.model().kit().view().set_focus ();
            break;
        }

        log () << "\tFinish step " << cursor.index() << "\n";
    }

    log () << "Finish protocol." << "\n";
    log () << "\n";
}

void Form_protocol::
stop ()
{
    Project::singleton().kit().protocol().cursor().current().stop ();
    stop_flag_ = true;
}

void Form_protocol::
tune ()
{
    Protocol & protocol    = Project::singleton().kit().protocol();
    protocol.set_model      (Project::singleton().current_model());
    Protocol_stage & stage = protocol.cursor().current();

    try
    {
        stage.tune ();
    }
    catch(Flaw & exception)
    {
        log () << "###\tFlaw: " << exception.message() << "\n";
        to_user().error (exception.message());
    }
    catch(...)
    {
        log () << "###\tUnknown exception!\n";
        to_user().error ("Unknown exception.");
    }
}

void Form_protocol::
start ()
{
    Own <Run_state> lock (new Run_state (*this));
    stop_flag_ = false;

    Model_kit::Lock model_lock = 
        Project::singleton().current_model().kit().lock();

    Protocol_cursor & cursor = Project::singleton().kit().protocol().cursor();
    Protocol_stage  & stage   = cursor.current();
 
    if (!stage.intention().check())
    {
        if (!to_user().question (stage.intention().last_message () 
                                + "\nDo you want to continue?"))
            return;
    }

    log() 
        << "\tStart step " << cursor.index() 
        << ": " << stage.intention().title()
        << " "  << stage.option ("Method")
        << " "  << stage.option ("Length")
        << " "  << stage.option ("T")
        << " "  << stage.option ("P")
        << " "  << stage.option ("Comment")
        << "\n";

    struct Process_events : public Command
    {
        void execute () {qApp->processEvents (QEventLoop::AllEvents, 100);}
    };
    stage.at_every_iteration (new Process_events);

    try
    {
        stage.execute();
    }
    catch(Flaw & exception)
    {
        log () << "###\tFlaw: " << exception.message() << "\n";
        to_user().error (exception.message());
    }
    catch(...)
    {
        log () << "###\tUnknown exception!\n";
        to_user().error ("Unknown exception.");
    }

    if (!stage.intention().was_reached())
        to_user().warning (stage.intention().last_message ());

    if (stop_flag_)
    {
        log () << "\t";
        log () << "User STOP.\n";
        Project::singleton().kit().protocol().model().kit().view().set_focus();
    }

    log () << "\tFinish step " << cursor.index() << "\n";
}


// Run_state ==================================================================
Form_protocol::Run_state * Form_protocol::run_state_ = 0;

Form_protocol::Run_state::
Run_state (Form_protocol & form)// : form_(form)
{
    apply (form);
}

void Form_protocol::Run_state::
apply (Form_protocol & form)
{
    form.pushButton_Run  ->              setEnabled (false);
    form.pushButton_Stop ->              setEnabled (true);
    form.pushButton_Tune ->              setEnabled (false);
    form.pushButton_Start->              setEnabled (false);
    run_state_ = this;

    qApp->processEvents (QEventLoop::AllEvents, 1);
}

Form_protocol::Run_state::
~Run_state()
{
    run_state_ = 0;

    if (Form_protocol::exist())
    {
        Form_protocol * form = Form_protocol::singleton();

        form->pushButton_Run  ->              setEnabled (true);
        form->pushButton_Stop ->              setEnabled (false);
        form->pushButton_Tune ->              setEnabled (true);
        form->pushButton_Start->              setEnabled (true);

        qApp->processEvents (QEventLoop::AllEvents, 1);
    }
}

}//MM
