#include "MD.h"

#include "Coordinates.h"
#include "Velocities.h"
#include "Force.h"
#include "Mass.h"
#include "MD_method.h"
#include "Defs.h"
//#include "MD_subject.h"

#include <math.h>

namespace MM
{
MD::
MD ()
:   
    step_length_(1e-015 * Constant::sec_to_time_unit),
    drift_length_(0.),
    kick_length_(0.),
    subject_ (0),
    method_(0)
{
}

MD::
MD (MD_subject & subject, MD_method & method)
:   
    step_length_(1e-015 * Constant::sec_to_time_unit),
    drift_length_(0.),
    kick_length_(0.),
    subject_ (&subject), method_(&method)
{
    method_->set_simulation (*this);
}


Text MD::
title () const
{
    return Text() + "MD: " + method_->title();
}

double MD::
step_length () const
{
    return step_length_;
}

void MD::
set_step_length (double new_length)
{
    step_length_ = new_length;
}

void MD::
set_method (MD_method & new_method)
{
    method_ = &new_method;
    method_->set_simulation (*this);
}

void MD::
set_subject (MD_subject & new_subject)
{
    subject_ = &new_subject;
}

void MD::
start ()
{
    algorithm().description (Text("start  ") + title());
    method_->start();
}

void MD::
cycle ()
{
    algorithm().description (Text("begin ") + iterations() + " step cycle");

    for (int i=0;  i<iterations();  ++i)
    {
        algorithm().description (Text(i));
        method_->step ();
        step_finished_.publish ();
    }
    algorithm().description ("end cycle");
}

int MD::
iterations () const
{
    return method_->iterations ();
}

void MD::
finish ()
{
    method_->finish ();
    algorithm().description (Text("finish ") + title());
    ENSURE ("", fabs(drift_length_ - kick_length_) < 0.000001 * drift_length_);
}

void MD::
kick (double h)
{
    kick_length_ += h * step_length_;
    subject_->kick (h*step_length_);
    algorithm().description
        (Text("kick    ") + h*step_length_ + "  \tsum = " + kick_length_);
}

void MD::
drift (double h)
{
    drift_length_ += h * step_length_;
    subject_->drift (h*step_length_);
    algorithm().description
        (Text("drift   ") + h*step_length_ + "  \tsum = " +drift_length_);
}
/*
bool MD::
invariant () const
{
    bool result = coordinats_ != 0 &&
                  velocities_ != 0 &&
                  forces_     != 0 &&
                  masses_     != 0;
    if (!result)    
        return false;

    int coordinats = coordinats_-> size ();
    int velocities = velocities_-> size ();
    int forces     = forces_->     size ();
    int masses     = masses_->     size ();

    result = coordinats == velocities &&
             coordinats == forces &&
             coordinats == masses;

    return result;
}//*/

}//MM









