#ifndef MD_SIMULATOR_H
#define MD_SIMULATOR_H

#ifndef OWN_H
#include "Own.h"
#endif

#ifndef ARRAY_OF_OWN_H
#include "Array_of_own.h"
#endif

#ifndef ARRAY_OF_REF_H
#include "Array_of_ref.h"
#endif

#ifndef INTERACTION_H
#include "Interaction.h"
#endif

#ifndef MD_H
#include "MD.h"
#endif

#ifndef MD_METHOD_H
#include "MD_method.h"
#endif

#ifndef MD_SUBJECT_H
#include "MD_subject.h"
#endif

#ifndef SIMULATOR_FRAME_H
#include "Simulator_frame.h"
#endif


namespace MM
{
class Prototype;

class MD_simulator : public Simulator_frame
{
    Own <MD_method>                     method_;
    Own <MD_subject>                    subject_;
    Own <MD>                            simulation_;

    double                              starting_temperature_;
    double                              starting_pressure_;
    double                              final_temperature_;
    double                              final_pressure_;

    static Array_of_own <MD_method>     method_prototype_;//fix remove
    static MD_simulator                 prototype_;

public:
    MD_simulator (Prototype &);
    MD_simulator ();
    virtual Simulator *     clone () const {return new MD_simulator;}

    explicit MD_simulator (Model &       model,
                           char const *  method_name = "Velocity_verlet");

    Text const&             title () const {static Text r="MD"; return r;}

    void                    set_method  (char const * name);
    void                    add_monitor (char const * name);

    void                    stop () {}//fix

    void                    tune  ();
    void                    start ();

    void                    edit_options (){}//fix

    bool                    support_duration () const                           {return true;}
//    double                  step_length () const;                // picoseconds
//    void                    set_step_length (double new_length); // picoseconds
    Duration_unit           step_length () const;
    void                    set_step_length (Duration_unit l);

    int                     steps     () const;
    void                    set_steps (int steps);

    bool                    support_temperature      () const                   {return true;}
    void                    setup_new_temperature    (double)                 {}//fix
    void                    set_starting_temperature (double v)                 {starting_temperature_=v;}//fix
    void                    set_starting_pressure    (double v)                 {starting_pressure_=v;}//fix
    void                    set_final_temperature    (double v)                 {final_temperature_=v;}
    void                    set_final_pressure       (double v)                 {final_pressure_=v;}
    bool                    support_thermostat       () const                   {return method_().support_thermostat ();}
    bool                    support_barostat         () const                   {return method_().support_barostat ();}
    bool                    has_thermostat           () const                   {return method_().has_thermostat ();}
    bool                    has_barostat             () const                   {return method_().has_barostat ();}
    void                    set_thermostat (bool on)                            {method_(). set_thermostat (on);}
    void                    set_barostat   (bool on)                            {method_(). set_barostat (on);}
    void                    after_each_step           (Command * to_do);
    void                    after_stop                (Command * to_do);
    void                    eliminate_after_each_step (Command & to_do);
    void                    eliminate_after_stop      (Command & to_do);
//    void                    unsubscribe (MD_monitor & to_do);    //fix

    void                    update ();
};

}//MM

#endif //MD_SIMULATOR_H
