#ifdef MM_WIN32
#pragma warning (disable : 4996)
#endif

#include "Form_geometry.h"

#include "Form_main_window.h"
#include "Qt_application.h"
#include "Model.h"
#include "Project.h"
#include "Project_kit.h"
#include "Point_3D_impl.h"
#include "Unit.h"
#include "User.h"

#include <QDoubleValidator>

namespace MM
{

Form_geometry::
Form_geometry ()
{
    setupUi (this);

    pushButton_Make_monitor->setEnabled (false);
    label_Manipulation_mode->setEnabled (false);
    comboBox_Manipulation_mode->setEnabled (false);

    for (int i=0;  i<Measure_geometry::type_counter(); ++i)
    {
        comboBox_1->insertItem (i, Measure_geometry::type_name(i).c_str());
        comboBox_2->insertItem (i, Measure_geometry::type_name(i).c_str());
        comboBox_3->insertItem (i, Measure_geometry::type_name(i).c_str());
        comboBox_4->insertItem (i, Measure_geometry::type_name(i).c_str());
    }

    connect (comboBox_1,        SIGNAL(currentIndexChanged (const QString &)), 
             this,              SLOT(set_type_1(const QString &)));
    
    connect (comboBox_2,        SIGNAL(currentIndexChanged (const QString &)), 
             this,              SLOT(set_type_2(const QString &)));
    
    connect (comboBox_3,        SIGNAL(currentIndexChanged (const QString &)), 
             this,              SLOT(set_type_3(const QString &)));
    
    connect (comboBox_4,        SIGNAL(currentIndexChanged (const QString &)), 
             this,              SLOT(set_type_4(const QString &)));

    connect (comboBox_Units,    SIGNAL(currentIndexChanged (const QString &)),
             this,              SLOT(set_units(const QString &)));
    

    //connect (spinBox_1, SIGNAL(editingFinished()), 
    //         this,      SLOT(set_number_1()));

    connect (spinBox_1,     SIGNAL(valueChanged (const QString &)), 
             this,          SLOT(set_number_1()));

    connect (spinBox_2,     SIGNAL(valueChanged (const QString &)), 
             this,          SLOT(set_number_2()));

    connect (spinBox_3,     SIGNAL(valueChanged (const QString &)), 
             this,          SLOT(set_number_3()));

    connect (spinBox_4,     SIGNAL(valueChanged (const QString &)), 
             this,          SLOT(set_number_4()));


    connect (checkBox_1,    SIGNAL(clicked (bool)), 
             this,          SLOT(set_active_1(bool)));

    connect (checkBox_2,    SIGNAL(clicked (bool)), 
             this,          SLOT(set_active_2(bool)));

    connect (checkBox_3,    SIGNAL(clicked (bool)), 
             this,          SLOT(set_active_3(bool)));

    connect (checkBox_4,    SIGNAL(clicked (bool)), 
             this,          SLOT(set_active_4(bool)));


    connect (toolButton_back,       SIGNAL(clicked(bool)),
             this,                  SLOT(prepend_point(bool)));

    connect (toolButton_forward,    SIGNAL(clicked(bool)),
             this,                  SLOT(append_point(bool)));


    connect (lineEdit_1,            SIGNAL(returnPressed()), 
             this,                  SLOT(set_position_1()));

    connect (lineEdit_2,            SIGNAL(returnPressed()), 
             this,                  SLOT(set_position_2()));

    connect (lineEdit_3,            SIGNAL(returnPressed()), 
             this,                  SLOT(set_position_3()));

    connect (lineEdit_4,            SIGNAL(returnPressed()), 
             this,                  SLOT(set_position_4()));

    //info_font_.setStyleHint (QFont::Courier);
    //textEdit->setCurrentFont (info_font_);

    lineEdit_New->setValidator (new QDoubleValidator(this));
    lineEdit_Add->setValidator (new QDoubleValidator(this));

    connect (lineEdit_New,          SIGNAL(editingFinished()),
             this,                  SLOT(new_parameter_value()));

    connect (lineEdit_Add,          SIGNAL(returnPressed()),
             this,                  SLOT(add_parameter_value()));

    
    //update_state ();

    //update_state_command_= create_command(*this, &Form_geometry::update_state);
    //Project::singleton().on_change_current (update_state_command_);
}

Form_geometry::
~Form_geometry ()
{
    //Project::singleton().on_change_current 
    //    (update_state_command_, false);
}

void Form_geometry::
showEvent (QShowEvent * event)
{
    if (Project::singleton().model_count() == 0)
        reset_current_model (Model::none());
    else
        reset_current_model (Project::singleton().current_model());

    QWidget::showEvent(event);
}

void Form_geometry::
reset_current_model (Model & model) 
{
    if (&model == &Model::none())
        setEnabled (false);
    else
    {
        setEnabled (true);
        update_state ();
    }
}

void Form_geometry::
pop_up ()
{
    Form_main_window &main_window 
        = Qt_application::singleton().main_window();

    if (Form_geometry::exist())
        main_window.pop_up (Form_geometry::singleton());
    else
        main_window.add_bottom_control
            (Form_geometry::singleton(), "Geometry");
}

void Form_geometry::
set_active_1 (bool on)  
{
    measure_geometry().set_active (0, on);
    update_state ();
}

void Form_geometry::
set_active_2 (bool on)  
{
    measure_geometry().set_active (1, on);
    update_state ();
}

void Form_geometry::
set_active_3 (bool on)  
{
    measure_geometry().set_active (2, on);
    update_state ();
}

void Form_geometry::
set_active_4 (bool on)  
{
    measure_geometry().set_active (3, on);
    update_state ();
}

void Form_geometry::
set_type_1 (const QString &type_name)  
{
    if (type_name == "Atom")
    {
        measure_geometry().set_atom_type (0, spinBox_1->value());
    }
    else  if (type_name == "Mass center")
    {
        measure_geometry().set_mass_center_type (0, spinBox_1->value());
    }
    else  if (type_name == "Space position")
    {
        Point_3D_impl position (0., 0., 0.);
        //fix
        measure_geometry().set_space_position_type (0, position);
    }
    else
        FLAW (Text("Wrong name - \"") + type_name.ascii() + "\".");
    //measure_geometry().set_active (3, on);
    update_state ();
}

void Form_geometry::
set_type_2 (const QString &type_name)  
{
    if (type_name == "Atom")
    {
        measure_geometry().set_atom_type (1, spinBox_1->value());
    }
    else  if (type_name == "Mass center")
    {
        measure_geometry().set_mass_center_type (1, spinBox_1->value());
    }
    else  if (type_name == "Space position")
    {
        Point_3D_impl position (0., 0., 0.);
        //fix
        measure_geometry().set_space_position_type (1, position);
    }
    else
        FLAW (Text("Wrong name - \"") + type_name.ascii() + "\".");
    //measure_geometry().set_active (3, on);
    update_state ();
}

void Form_geometry::
set_type_3 (const QString &type_name)  
{
    if (type_name == "Atom")
    {
        measure_geometry().set_atom_type (2, spinBox_1->value());
    }
    else  if (type_name == "Mass center")
    {
        measure_geometry().set_mass_center_type (2, spinBox_1->value());
    }
    else  if (type_name == "Space position")
    {
        Point_3D_impl position (0., 0., 0.);
        //fix
        measure_geometry().set_space_position_type (2, position);
    }
    else
        FLAW (Text("Wrong name - \"") + type_name.ascii() + "\".");
    //measure_geometry().set_active (3, on);
    update_state ();
}

void Form_geometry::
set_type_4 (const QString &type_name)  
{
    if (type_name == "Atom")
    {
        measure_geometry().set_atom_type (3, spinBox_1->value());
    }
    else  if (type_name == "Mass center")
    {
        measure_geometry().set_mass_center_type (3, spinBox_1->value());
    }
    else  if (type_name == "Space position")
    {
        Point_3D_impl position (0., 0., 0.);
        //fix
        measure_geometry().set_space_position_type (3, position);
    }
    else
        FLAW (Text("Wrong name - \"") + type_name.ascii() + "\".");
    //measure_geometry().set_active (3, on);
    update_state ();
}

void Form_geometry::
set_units(const QString &type_name)  
{
    Measure_geometry & geometry = measure_geometry();
    Unit const &       parameter = geometry.parameter();
    parameter.set_current_type (type_name.ascii());
    update_parameter ();
}

void Form_geometry::
set_number_1 () 
{
    measure_geometry().set_number (0, spinBox_1->value());
    update_state ();
}

void Form_geometry::
set_number_2 () 
{
    measure_geometry().set_number (1, spinBox_2->value());
    update_state ();
}

void Form_geometry::
set_number_3 () 
{
    measure_geometry().set_number (2, spinBox_3->value());
    update_state ();
}

void Form_geometry::
set_number_4 () 
{
    measure_geometry().set_number (3, spinBox_4->value());
    update_state ();
}

void Form_geometry::
prepend_point (bool)
{
    measure_geometry().prepend_point();
    update_state ();
}

void Form_geometry::
append_point (bool)
{
    measure_geometry().append_point();
    update_state ();
}

void Form_geometry::set_position_1 () {set_position (0, lineEdit_1->text().ascii());}
void Form_geometry::set_position_2 () {set_position (1, lineEdit_2->text().ascii());}
void Form_geometry::set_position_3 () {set_position (2, lineEdit_3->text().ascii());}
void Form_geometry::set_position_4 () {set_position (3, lineEdit_4->text().ascii());}

void Form_geometry::
set_position (int point, const char * text)
{
    double x, y, z;
    int result = sscanf (text, "%lg%lg%lg", &x, &y, &z);
    if (result == 3)
    {
        measure_geometry().set_space_position (point, Point_3D_impl (x,y,z));
    }
    else
        to_user (). warning (Text(text) + " - wrong string.\n"
                            "Sould be three number (x y z).");

    update_state ();
}

void Form_geometry::
update_info ()
{
    //textEdit->setPlainText(spinBox_1->text());//fix remove

    Text info;
    measure_geometry().get_info (info);
    //textEdit->setFontFamily ("Courier");
    textEdit->setPlainText (info.c_str());

    update_parameter ();
}

void Form_geometry::
update_parameter ()
{
    Measure_geometry & geometry = measure_geometry();

    lineEdit_New->      blockSignals (true);
    groupBox_Parameter->blockSignals (true);
    comboBox_Units->    blockSignals (true);

    Text const & type_name = geometry.parameter_type_name();
    groupBox_Parameter->setTitle (type_name.c_str());

    comboBox_Units->clear ();

    //lineEdit_Old->setText ();
    QString str;

    if (type_name == "No parameters")
        lineEdit_New->setText ("");
    else
    {
        Unit const & parameter = geometry.parameter();

        str.setNum (parameter.value (), 'g', 14);
        lineEdit_New->setText (str);

        int type_count = parameter.type_count();
        
        for (int i=0;  i<type_count;  ++i)
            comboBox_Units->insertItem (i, parameter.type(i).c_str());

        int index = comboBox_Units->findText(parameter.current_type().c_str());
        comboBox_Units->setCurrentIndex (index);
    }

    lineEdit_New->      blockSignals (false);
    groupBox_Parameter->blockSignals (false);
    comboBox_Units->    blockSignals (false);
}

void Form_geometry::
update_state ()
{
    //static bool intrance = false;
    //if (intrance)                   //fix remove
    //    return;
    //intrance = true;

    update_info ();

    Measure_geometry &ref = measure_geometry ();

    checkBox_1->blockSignals (true);
    checkBox_2->blockSignals (true);
    checkBox_3->blockSignals (true);
    checkBox_4->blockSignals (true);

    if (ref.is_active (0))
    {
        checkBox_1->setCheckState (Qt::Checked);
    }
    else
    {
        checkBox_1->setCheckState (Qt::Unchecked);
    }

    if (ref.is_active (1))
    {
        checkBox_2->setCheckState (Qt::Checked);
    }
    else
    {
        checkBox_2->setCheckState (Qt::Unchecked);
    }

    if (ref.is_active (2))
    {
        checkBox_3->setCheckState (Qt::Checked);
    }
    else
    {
        checkBox_3->setCheckState (Qt::Unchecked);
    }

    if (ref.is_active (3))
    {
        checkBox_4->setCheckState (Qt::Checked);
    }
    else
    {
        checkBox_4->setCheckState (Qt::Unchecked);
    }
    checkBox_1->blockSignals (false);
    checkBox_2->blockSignals (false);
    checkBox_3->blockSignals (false);
    checkBox_4->blockSignals (false);

    int index;
    index = comboBox_1->findText (measure_geometry ().type(0).c_str());
    comboBox_1->blockSignals (true);
    comboBox_1->setCurrentIndex (index);
    comboBox_1->blockSignals (false);
    index = comboBox_2->findText (measure_geometry ().type(1).c_str());
    comboBox_2->blockSignals (true);
    comboBox_2->setCurrentIndex (index);
    comboBox_2->blockSignals (false);
    index = comboBox_3->findText (measure_geometry ().type(2).c_str());
    comboBox_3->blockSignals (true);
    comboBox_3->setCurrentIndex (index);
    comboBox_3->blockSignals (false);
    index = comboBox_4->findText (measure_geometry ().type(3).c_str());
    comboBox_4->blockSignals (true);
    comboBox_4->setCurrentIndex (index);
    comboBox_4->blockSignals (false);

    spinBox_1->blockSignals (true);
    spinBox_2->blockSignals (true);
    spinBox_3->blockSignals (true);
    spinBox_4->blockSignals (true);
    spinBox_1->setValue (measure_geometry ().number(0));
    spinBox_2->setValue (measure_geometry ().number(1));
    spinBox_3->setValue (measure_geometry ().number(2));
    spinBox_4->setValue (measure_geometry ().number(3));
    spinBox_1->blockSignals (false);
    spinBox_2->blockSignals (false);
    spinBox_3->blockSignals (false);
    spinBox_4->blockSignals (false);

    lineEdit_1->blockSignals (true);
    lineEdit_2->blockSignals (true);
    lineEdit_3->blockSignals (true);
    lineEdit_4->blockSignals (true);
    
    Point_3D_impl position (0,0,0);
    QString       text;
    
    measure_geometry ().get_space_position (0, position);
    text = QString("%1 %2 %3")
        .arg(position.x()).arg(position.y()).arg(position.z());
    lineEdit_1->setText (text);

    measure_geometry ().get_space_position (1, position);
    text = QString("%1 %2 %3")
        .arg(position.x()).arg(position.y()).arg(position.z());
    lineEdit_2->setText (text);

    measure_geometry ().get_space_position (2, position);
    text = QString("%1 %2 %3")
        .arg(position.x()).arg(position.y()).arg(position.z());
    lineEdit_3->setText (text);

    measure_geometry ().get_space_position (3, position);
    text = QString("%1 %2 %3")
        .arg(position.x()).arg(position.y()).arg(position.z());
    lineEdit_4->setText (text);

    lineEdit_1->blockSignals (false);
    lineEdit_2->blockSignals (false);
    lineEdit_3->blockSignals (false);
    lineEdit_4->blockSignals (false);

    //intrance = false;
}

void Form_geometry::
new_parameter_value ()
{
    Measure_geometry & geometry = measure_geometry ();

    

    Own <Unit> parameter (geometry.parameter().clone());
    parameter().set_value (lineEdit_New->text().toDouble());
    geometry.set_parameter (parameter());
    update_state ();
}

void Form_geometry::
add_parameter_value ()
{
    Measure_geometry & geometry = measure_geometry ();

    Own <Unit> parameter (geometry.parameter().clone());
    parameter().set_value (lineEdit_Add->text().toDouble());
    geometry.add_parameter (parameter());
    update_state ();
}

Measure_geometry & Form_geometry::
measure_geometry ()
{
    return Project::singleton().kit().measure_geometry();
}

}//MM
