#ifndef FORM_SEQUENCE_EDITOR_H
#define FORM_SEQUENCE_EDITOR_H

#ifndef UI_FORM_SEQUENCE_EDITOR_H
#include "ui_Form_sequence_editor.h"
#endif

#ifndef DYNAMIC_SINGLETON_H
#include "Dynamic_singleton.h"
#endif

#ifndef CHAIN_GROUP_H
#include "Chain_group_impl.h"
#endif

#ifndef MODEL_SINGLETON_VIEW_H
#include "Model_singleton_view.h"
#endif

#ifndef MOLMECCANO_FILE_H
#include "MolMeccano_file.h"
#endif

#ifndef MODEL_IMPL_H
#include "Model_impl.h"
#endif

#include <QDir>

class QListWidgetItem;

namespace MM
{
class Sequence_model;

class Form_sequence_editor
:
    public QWidget,
    public Ui::Form_sequence_editor,
    public Dynamic_singleton <Form_sequence_editor>,
    public Model_singleton_view
{
    Q_OBJECT

    friend class Dynamic_singleton <Form_sequence_editor>;
    Form_sequence_editor ();

    QDir                repository_dir_;
    QGridLayout *       residues_layout_;
    QButtonGroup *      button_group_;

    Sequence_model *    sequence_model_;
    Model_impl          stub_model_;
    MolMeccano_file     conformational_type_;
    int                 current_conformation_n_;

    int                 mol_n_;
    int                 res_n_;

    Chain_group_impl    chain_;

public:

    static  void        pop_up ();

    void                reset_current_model (Model &);
    void                reset (Model_event::Hint);

public slots:
    virtual void        new_sequence    ();
    virtual void        delete_sequence ();
    virtual void        show_details    ();

    void                change_dir (QListWidgetItem * item);

    void                select_residue (const QModelIndex & index);
    void                choose_residue (Text const & name);
    void                apply                       ();
    void                new_molecule                ();

protected slots:
    void                update_form                 ()  {update_class ();}
    void                update_class                ();
    void                update_residues             ();
    void                update_conformational_type  ();

    void                flush_current_conformation  ();
    void                change_current_conformation ();
    void                update_current_conformation ();
    
    void                update_geometry_parameters  ();

protected:
    void                showEvent (QShowEvent * event);

    QDir                residues_directory ();
};

}//MM

#endif //FORM_SEQUENCE_EDITOR_H
