#include "Form_main_window.h"

#include "Form_model.h"
#include "Flaw.h"
#include "Path.h"

#include <QFont>
#include <QToolBar>
#include <QSplitter>
#include <QTabBar>
#include <QWorkspace>
#include <QTabWidget>
#include <QMenuBar>
#include <QLabel>
#include <QStatusBar>

#include "ui_Form_bottom_tab_close.h"
#include "ui_Form_left_tab_close.h"

namespace MM
{

Form_main_window::
//Form_main_window (QWidget* parent, const char* name)
//Form_main_window (QWidget* parent, const char*)
//:   QMainWindow (parent)
Form_main_window ()
{
//return;
    #ifdef MM_WIN32
    QFont new_font (font());    
    new_font.setBold (TRUE);    
    new_font.setPointSize (8);    
    setFont (new_font);
    menuBar()->setFont (new_font);
    #endif

    //-------------------------------------------------------------------------

    status_label = new QLabel ("Status bar");

    status_label->setPaletteBackgroundColor (QColor(230,230,230));
    status_label->setAlignment(Qt::AlignHCenter);
    #ifdef MM_WIN32
    status_label->setFont (new_font);
    #endif
    statusBar()->addWidget (status_label, 1, true); 
         
    setName ("MolMeccano");

    //-------------------------------------------------------------------------

    setCentralWidget (new QWidget(this, "qt_central_widget"));

    layout = new QVBoxLayout (centralWidget());
    layout->setSpacing (1);
    layout->setMargin  (0);

    tab_bar = new QTabBar (centralWidget());
    tab_bar->setShape (QTabBar:: TriangularNorth);

    layout->addWidget (tab_bar);

    //-------------------------------------------------------------------------
    workspace = new QWorkspace ();
 
    double k = 0.7;
    workspace->setBackground (QBrush (QColor (int(k*205), int(k*205), int(k*192))));
    //workspace->setMinimumSize (0, 0);
    //workspace->setSizePolicy (QSizePolicy::Ignored, QSizePolicy::Ignored);
    //setMinimumSize (0, 0);

    tab_right = new QTabWidget ();
    tab_right->setTabPosition (QTabWidget::Bottom);
    tab_right->setTabShape    (QTabWidget::Triangular);
    tab_right->setMinimumSize (0, 0);

    horizontal_right_splitter = new QSplitter();
    horizontal_right_splitter->setFrameShape( QSplitter::NoFrame );
    horizontal_right_splitter->setOrientation( Qt::Horizontal );
    horizontal_right_splitter->setOpaqueResize (true);
    horizontal_right_splitter->setHandleWidth (4);

    horizontal_right_splitter->addWidget (workspace);
    horizontal_right_splitter->addWidget (tab_right);
    
    tab_bottom  = new QTabWidget ();
    tab_bottom->setTabPosition (QTabWidget::Bottom);
    tab_bottom->setTabShape    (QTabWidget::Triangular);

    vertical_splitter = new QSplitter ();
    vertical_splitter->setFrameShape( QSplitter::NoFrame );
    vertical_splitter->setOrientation( Qt::Vertical );
    vertical_splitter->setOpaqueResize (true);
    vertical_splitter->setHandleWidth (4);

    vertical_splitter->addWidget (horizontal_right_splitter);
    vertical_splitter->addWidget (tab_bottom);

    tab_left    = new QTabWidget ();
    tab_left->setTabPosition (QTabWidget::Bottom);
    tab_left->setTabShape    (QTabWidget::Triangular);

    horizontal_left_splitter = new QSplitter ();
    horizontal_left_splitter->setFrameShape( QSplitter::NoFrame );
    horizontal_left_splitter->setOrientation( Qt::Horizontal );
    horizontal_left_splitter->setOpaqueResize (true);
    horizontal_left_splitter->setHandleWidth (4);

    horizontal_left_splitter->addWidget (tab_left);
    horizontal_left_splitter->addWidget (vertical_splitter);

    layout->addWidget (horizontal_left_splitter);


//    connect (tab_left,      SIGNAL(currentChanged (QWidget *)), 
//             this,          SLOT(update_left_panel(QWidget *)));

    connect (tab_bar,       SIGNAL(selected(int)), 
             this,          SLOT(select_window(int)));

    connect (workspace,     SIGNAL(windowActivated (QWidget *)), 
             this,          SLOT(select_tab(QWidget *)));

    line_up ();
//    showMaximized ();
}

Form_main_window::~Form_main_window()
{
}

//void Form_main_window::
//update_left_panel (QWidget * w)
//{
//    w->update();
//}

void Form_main_window::
set_status (const QString &text) 
{
    status_label->setText (text);
    status_label->repaint ();
}

void Form_main_window::
select_window (int n)
{
    //workspace->windowList (QWorkspace::CreationOrder).take (index)->
    //    setFocus ();

    int identifier = tab_bar->tabData(n).toInt();

    QWidgetList list = workspace->windowList (QWorkspace::CreationOrder);
    for (int i=0;  i<list.count();  ++i)
    {
        if(((Form_model *)(list.at (i)))->identifier() == identifier)
        {
            list.at (i)->setFocus ();
            //list.at (i)->repaint (); //fix
            return;
        }
    }
    //3 FLAW ("Window not found.");
}

void Form_main_window::
select_tab (QWidget * widget)
{
    //int index = workspace->windowList(QWorkspace::CreationOrder).find(widget);
    //tab_bar->setCurrentTab (index);
    if (widget == 0) return;
    //tab_bar->setCurrentTab (((Form_model *)widget)->identifier());

    int id = ((Form_model *)widget)->identifier();

    for (int i=0;  i<tab_bar->count();  ++i)
        if (tab_bar->tabData(i).toInt() == id)
        {
            tab_bar->setCurrentTab (i);
            return;
        }

    FLAW ("Can not find appopriate tab_bar.");
}

void Form_main_window::
add_MDI_window (Form_model *render_area, const char *name)
{
    static int id = 0;
    ++id;//fix to model ID

    render_area->set_main_window (this);
    workspace->addWindow (render_area);


    //3 QTab *new_tab = new QTab(name);
    //tab_bar->addTab (new_tab);
    int index = tab_bar->addTab (name);
    tab_bar->setTabData (index, id);
    //tab_bar->setCurrentTab (new_tab);
    //tab_bar->show();
    tab_bar->update ();

    //render_area->set_index (tab_bar->currentTab ());
    //3 render_area->set_identifier (new_tab->identifier ());
    render_area->set_identifier (id);
    render_area->show();
}

void Form_main_window::
remove_render_area (int id)
{
    //3 QTab *tab = tab_bar->tab (index);
    //3 tab_bar->removeTab (tab);
	
    for (int i=0;  i<tab_bar->count();  ++i)
        if (tab_bar->tabData(i).toInt() == id)
        {
            tab_bar->removeTab (i);
            return;
        }

    FLAW ("Can not find appopriate tab_bar.");
}

void Form_main_window::
add_left_control (QWidget *control, const QString & label)
{
    QWidget *vbox = new QWidget;
    QVBoxLayout *layout = new QVBoxLayout;

    QWidget *form_close = new QWidget;
    Ui::Form_left_tab_close ui;
    ui.setupUi (form_close);
    layout->addWidget (form_close);
    layout->addWidget (control);

    vbox->setLayout (layout);

    tab_left->addTab   (vbox, label);
    tab_left->showPage (vbox);

    QToolButton *toolButton_close = qFindChild <QToolButton *> (form_close, "toolButton_close");
    connect (toolButton_close,  SIGNAL(clicked()), 
             this,              SLOT(remove_left_control()));
    toolButton_close->setIcon(QIcon((
        MM::Path::icon() + "close_cross.png").c_str()));

    line_up ();
}

void Form_main_window::
add_right_control (QWidget *control, const QString & label)
{
    QWidget *vbox = new QWidget;
    QVBoxLayout *layout = new QVBoxLayout;

    QWidget *form_close = new QWidget;
    Ui::Form_left_tab_close ui;
    ui.setupUi (form_close);
    layout->addWidget (form_close);
    layout->addWidget (control);

    vbox->setLayout (layout);

    tab_right->addTab   (vbox, label);
    tab_right->showPage (vbox);

    QToolButton *toolButton_close = qFindChild <QToolButton *> (form_close, "toolButton_close");
    connect (toolButton_close,  SIGNAL(clicked()), 
             this,              SLOT(remove_right_control()));
    toolButton_close->setIcon(QIcon((
        MM::Path::icon() + "close_cross.png").c_str()));

    line_up ();
}

void Form_main_window::
add_bottom_control (QWidget *control, const QString & label)
{
    QWidget *hbox = new QWidget;
    QHBoxLayout *layout = new QHBoxLayout;
    layout->addWidget (control);

    QWidget *form_close = new QWidget;
    Ui::Form_bottom_tab_close ui;
    ui.setupUi (form_close);
    layout->addWidget (form_close);
    
    hbox->setLayout (layout);

    tab_bottom->addTab   (hbox, label);
    tab_bottom->showPage (hbox);

    QToolButton *toolButton_close = qFindChild <QToolButton *> (form_close, "toolButton_close");
    connect (toolButton_close,  SIGNAL(clicked()), 
             this,              SLOT(remove_bottom_control()));
    toolButton_close->setIcon(QIcon((
        MM::Path::icon() + "close_cross.png").c_str()));

    line_up ();
    //tab_bottom_->addTab   (control, label);
    //tab_bottom_->showPage (control);

    /*QString status;
    QValueList<int> list = horizontal_splitter_->sizes();
    QValueList<int>::Iterator it = list.begin();
    while( it != list.end() ) {
        status.append (QString("QValueList %1   ").arg(*it));
        ++it;
    }
    set_status (status);//*/
}

void Form_main_window::
pop_up (QWidget *control)
{
    tab_bottom->showPage (control->parentWidget());
    tab_left->  showPage (control->parentWidget());
    //line_up ();
}

void Form_main_window::
remove_left_control ()
{
    delete tab_left->currentPage ();
    line_up ();
}

void Form_main_window::
remove_right_control ()
{
    delete tab_right->currentPage ();
    line_up ();
}

void Form_main_window::
remove_bottom_control ()
{
    QWidget *layout = tab_bottom->currentPage ();
    //tab_bottom_->removePage (layout);
    delete layout;
    line_up ();
}

void Form_main_window::
line_up ()
{
    /*QList<int> vertical_size         = vertical_splitter        ->sizes();
    QList<int> horizontal_left_size  = horizontal_left_splitter ->sizes();
    QList<int> horizontal_right_size = horizontal_right_splitter->sizes();//*/

    QList<int> new_vertical_size;
    QList<int> new_horizontal_left_size;
    QList<int> new_horizontal_right_size;

    /*int vertical_sum         = vertical_size.first() 
                             + vertical_size.last(); 

    int horizontal_left_sum  = horizontal_left_size. first() 
                             + horizontal_left_size. last(); 
    
    int horizontal_right_sum = horizontal_right_size.first() 
                             + horizontal_right_size.last();//*/
    

    if (tab_left->count() == 0)
    {
        new_horizontal_left_size.push_back (0);
        //3 new_horizontal_size.push_back (horizontal_sum);
        new_horizontal_left_size.push_back (workspace->width());
    }
    else
    {
        new_horizontal_left_size.push_back (10);
        new_horizontal_left_size.push_back (workspace->width() - 10);
    }
    horizontal_left_splitter->setSizes (new_horizontal_left_size);
    
    if (tab_right->count() == 0)
    {
        new_horizontal_right_size.push_back (workspace->width());
        new_horizontal_right_size.push_back (0);
    }
    else
    {
        new_horizontal_right_size.push_back (workspace->width() - 10);
        new_horizontal_right_size.push_back (10);
    }
    horizontal_right_splitter->setSizes (new_horizontal_right_size);
    
    
    if (tab_bottom->count() == 0)
    {
        new_vertical_size.push_back (workspace->height());
        new_vertical_size.push_back (0);
    }
    else
    {
        new_vertical_size.push_back (workspace->height() - 10);
        new_vertical_size.push_back (10);
    }
    vertical_splitter->setSizes (new_vertical_size);
    
/*

    if (tab_bottom->count() == 0)
    {
        new_vertical_size.push_back (vertical_sum);
        new_vertical_size.push_back (0);
    }
    else
    {
        new_vertical_size.push_back (vertical_sum - 10);
        new_vertical_size.push_back (10);
    }
    vertical_splitter->setSizes (new_vertical_size);//*/
}

void Form_main_window::
add_to_menu (const QString & menu_name, QAction * action)
{
    for (int i=0;  i<menu_.size();  ++i)
    {
        QString title = menu_[i]->title();

        if (title == menu_name)
        {
            menu_[i]->addAction (action);
            return;
        }
    }

    QMenu *new_menu = menuBar()->addMenu (menu_name);
    new_menu->addAction (action);
    menu_.push_back (new_menu);
}

void Form_main_window::
insert_menu_separator (const QString & menu_name)
{
    for (int i=0;  i<menu_.size();  ++i)
    {
        if (menu_[i]->title() == menu_name)
        {
            menu_[i]->addSeparator ();
            return;
        }
    }

    QMenu *new_menu = menuBar()->addMenu (menu_name);
    new_menu->addSeparator ();
    menu_.push_back (new_menu);
}

QToolBar * Form_main_window::
tool_bar (QString const& title)
{
    for (int i=0;  i<tool_bar_.size();  ++i)
    {
        if (tool_bar_[i]->windowTitle() == title)
            return tool_bar_[i];
    }

    QToolBar *new_tool_bar = addToolBar (title);
    tool_bar_.push_back (new_tool_bar);
    return new_tool_bar;
}

QMenu * Form_main_window::
menu (QString const& title)
{
    for (int i=0;  i<menu_.size();  ++i)
    {
        if (menu_[i]->title() == title)
            return menu_[i];
    }

    QMenu *new_menu = menuBar()->addMenu (title);
    menu_.push_back (new_menu);
    return new_menu;
}

void Form_main_window::
remove_empty_menus ()
{
    QMenuBar *menu = menuBar();

    for (int i=menu->actions ().count() - 1;  i>=0;  --i)
    {
        if (menu->actions().at(i)->menu()->actions().count() == 0)
            menu->removeAction (menu->actions().at (i));
    }
}
void Form_main_window::
remove_empty_tool_bars ()
{
    for (int i = tool_bar_.size() - 1;  i>=0;  --i)
    {
        int n = tool_bar_[i]->actions().count();

        if (n == 0)
        {
            removeToolBar (tool_bar_[i]);
            delete tool_bar_[i];
            tool_bar_.erase (tool_bar_.begin() + i);
        }
        else
            tool_bar_[i]->addWidget (new QLabel("    "));
    }
}

}//MM
