#include "QRMWidget.h"
//Added by qt3to4:
#include <QMouseEvent>
#include <QTimerEvent>
#include <QEvent>
#include <QPaintEvent>
#include <QCloseEvent>

#include "Log.h"
#include "User.h"
#include "Debug.h"

#include "Model.h"
#include "User.h"
#include "Text.h"

#include "Flaw.h"
#include "User.h"
#include "OpenRM_model_view.h"
#include "Timer.h"

#include <rm/rm.h>
#include <rm/rmaux.h>

#include <qmessagebox.h>
#include <qaction.h>
#include <qdatetime.h>
#include <qapplication.h>

#include <Q3Action>

#include <math.h>

#if defined(Q_WS_X11) || defined(Q_WS_MAC)
#include <GL/glx.h>
#endif

//#include <qapplication.h> 

extern "C" float     RM_DEFAULT_SPECULAR_EXPONENT;
extern "C" RMcolor4D RM_DEFAULT_AMBIENT_COLOR,
                     RM_DEFAULT_DIFFUSE_COLOR, 
                     RM_DEFAULT_SPECULAR_COLOR, 
                     RM_DEFAULT_UNLIT_COLOR;

#define max(a, b)  (((a) > (b)) ? (a) : (b))
#define min(a, b)  (((a) < (b)) ? (a) : (b))


/*inline float 
pixel_to_float (float pixel, int dimension, int scale)
{
    return ((pixel - .5 * (dimension - scale)) - (scale >> 1)) / (scale >> 1);
}//*/

inline float 
pixel_to_float (float pixel, int dimension, int scale)
{
    return (2. * pixel - dimension) / scale;
}//*/

const double mouse_scaling_increment = 1.;

QRMWidget::QRMWidget ()
:
//    model_view_ (0),
    pipe_       (0),
    root_node_  (0),
    camera_node_(0),
    scene_node_ (0),
    object_node_(0)
{
    setFocusPolicy (Qt::ClickFocus);

    Q3Action *fit_action = new Q3Action ("Fit", QObject::tr(" "), this);
    connect (fit_action,    SIGNAL (activated()), 
             this,          SLOT   (fit_to_screen()));

    //idle_timer_id_ = startTimer (0);
}

/*QRMWidget::QRMWidget (QWidget *parent, const char *name)
//:   QGLWidget (parent, name),
:   QGLWidget (0, name),
//    model_view_ (0),
    pipe_       (0),
    root_node_  (0),
    camera_node_(0),
    scene_node_ (0),
    object_node_(0)
{
    setFocusPolicy (Qt::ClickFocus);

    Q3Action *fit_action = new Q3Action ("Fit", QObject::tr(" "), this);
    connect (fit_action,    SIGNAL (activated()), 
             this,          SLOT   (fit_to_screen()));

    //idle_timer_id_ = startTimer (0);
}//*/

QRMWidget::
~QRMWidget()
{
    destroy ();       //fix !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

//    model_view_ = 0;
    pipe_       = 0;
    root_node_  = 0;
    camera_node_= 0;
    scene_node_ = 0;
    object_node_= 0;
}

void QRMWidget::
destroy (bool destroyWindow, bool destroySubWindows)
{
    //MM::log () << " destroy ";
    //MM::log () << model_view_->model().atom_count() << " ";
///    model_view_->erase(); //fix
//    MM::log () << model_view_->model().atom_count() << " ";

    RMenum result = RM_WHACKED;
    
    result = rmSubTreeDelete (root_node_);
    if (result != RM_CHILL)
        FLAW ("rmSubTreeDelete");//*/

    // wes 9/26/04 - it is correct to first close the pipe with rmPipeClose
    // followed by deleting it with rmPipeDelete. Alternatively, you could
    // just call rmPipeDelete with no rmPipeClose, since rmPipeDelete will
    // itself call rmPipeClose. Having both here was a good exercise for
    // OpenRM for it revealed a bug. (Thanks!)

    //fix !!!

    /*result = rmPipeClose  (pipe_);
    if (result != RM_CHILL)
        FLAW ("rmPipeClose");//*/
    
    /*result = rmPipeDelete (pipe_);
    if (result != RM_CHILL)
        FLAW ("rmPipeDelete");//*/
	
    QGLWidget::destroy (destroyWindow, destroySubWindows);
}

bool QRMWidget::
invariant () const
{
    return 
        //model_view_  != 0 && 
        pipe_        != 0
        && root_node_   != 0
        //&& camera_node_ != 0
        && scene_node_  != 0
        && object_node_ != 0;
}

void QRMWidget::
set_model_view (MM::OpenRM_model_view *model_view)
{
    model_view_ = model_view;
    //INVARIANT;
    CHECK ("", model_view_  != 0);
}

void QRMWidget::
paintEvent (QPaintEvent *event)
{
    //PP
    INVARIANT;

    RMtime *time_1 = rmTimeNew ();
    RMtime *time_2 = rmTimeNew ();
    rmTimeCurrent (time_1);

    QGLWidget::paintEvent(event);

    rmTimeCurrent (time_2);
    //MM::to_user().status (MM::Text(rmTimeDifferenceMS (time_1, time_2)) + " Milliseconds");
    rmTimeDelete (time_1);
    rmTimeDelete (time_2);

    
    INVARIANT;
    //PP
}

void QRMWidget::
initializeGL ()
{
    RMenum result = RM_WHACKED;

    // Pipe
    #if defined(Q_WS_WIN)
        pipe_ = rmPipeNew (RM_PIPE_WGL);
    #else
        pipe_ = rmPipeNew (RM_PIPE_GLX);
    #endif

    if (pipe_ == 0)
        FLAW ("Can not create RMpipe.");

    result = rmPipeSetProcessingMode  (pipe_, RM_PIPE_MULTISTAGE);
    if (result != RM_CHILL)
        FLAW ("rmPipeSetProcessingMode");

    rmPipeSetSwapBuffersFunc (pipe_, NULL);

#if 0
    // wes 9/26/04 - don't set the window handle, just tell OpenRM how
    // big the display window is in terms of width, height.
    result = rmPipeSetWindow          (pipe_, winId(), width(), height());
#else
    result = rmPipeSetWindowSize(pipe_, width(), height());
#endif
    if (result != RM_CHILL)
        FLAW ("rmPipeSetWindow");

    //rmxPipeSetDisplay (pipe_, NULL);
    rmPipeSetContext  (pipe_, NULL);

    result = rmPipeMakeCurrent (pipe_);  //fix move to render ?
    if (result != RM_CHILL)
        FLAW ("rmPipeMakeCurrent");//*/

    // Scene graph
    
    // initObjectTree /////////////////////////////////////////////////////////////////
    //root_node_   = rmNodeNew ("Root",   RM_RENDERPASS_3D, RM_RENDERPASS_OPAQUE);
    float      vp[4];
    RMvertex3D bmin, bmax;
    
    root_node_ = rmNodeNew("Root", RM_RENDERPASS_3D, RM_RENDERPASS_OPAQUE);
    //root_node_ = rmNodeNew("Root", RM_RENDERPASS_ALL, RM_RENDERPASS_ALL);//*/

    vp[0] = RM_DEFAULT_VIEWPORT_XMIN;
    vp[1] = RM_DEFAULT_VIEWPORT_YMIN;
    vp[2] = RM_DEFAULT_VIEWPORT_XMAX;
    vp[3] = RM_DEFAULT_VIEWPORT_YMAX;

    rmNodeSetSceneViewport(root_node_, vp);

    bmin.x = bmin.y = bmin.z = RM_MAXFLOAT;
    bmax.x = bmax.y = bmax.z = RM_MINFLOAT;
    rmNodeSetBoundingBox (root_node_, &bmin, &bmax);

    rmNodeSetAmbientColor(root_node_, &RM_DEFAULT_AMBIENT_COLOR);
    rmNodeSetDiffuseColor(root_node_, &RM_DEFAULT_DIFFUSE_COLOR);
    rmNodeSetSpecularColor(root_node_, &RM_DEFAULT_SPECULAR_COLOR);
    rmNodeSetSpecularExponent(root_node_, RM_DEFAULT_SPECULAR_EXPONENT);
    rmNodeSetUnlitColor(root_node_, &RM_DEFAULT_UNLIT_COLOR);
    rmNodeSetNormalizeNormals(root_node_, RM_FALSE);

    /* set the default rendering parms */
    rmNodeSetShader(root_node_, RM_DEFAULT_RENDERMODE); /* RM_RENDERMODE_SMOOTH */

    rmNodeSetPolygonDrawMode(root_node_, RM_FRONT_AND_BACK, RM_FILL);

    //rmNodeSetPolygonCullMode(root_node_, RM_CULL_NONE);
    rmNodeSetPolygonCullMode(root_node_, RM_CULL_BACK);
    rmNodeSetFrontFace(root_node_, RM_CCW);

    rmNodeSetLineStyle(root_node_, RM_LINES_SOLID);
    rmNodeSetLineWidth(root_node_, RM_LINEWIDTH_NARROW);

    ///////////////////////////////////////////////////////////////////////////////////

    //camera_node_ = rmNodeNew ("Camera", RM_RENDERPASS_3D, RM_RENDERPASS_OPAQUE);
    //scene_node_  = rmNodeNew ("Scene",  RM_RENDERPASS_3D, RM_RENDERPASS_OPAQUE);
    camera_node_ = rmNodeNew ("Camera", RM_RENDERPASS_ALL, RM_RENDERPASS_ALL);
    scene_node_  = rmNodeNew ("Scene",  RM_RENDERPASS_ALL, RM_RENDERPASS_ALL);
    rmNodeAddChild (root_node_,   camera_node_);
    rmNodeAddChild (camera_node_, scene_node_);

    RMmatrix initial_transform;
    rmMatrixIdentity (&initial_transform);
    rmNodeSetRotateMatrix (root_node_,   &initial_transform);
    rmNodeSetRotateMatrix (camera_node_, &initial_transform);
    rmNodeSetRotateMatrix (scene_node_,  &initial_transform);

    float viewport[4] = {0.F, 0.F, 1.F, 1.F};
    rmNodeSetSceneViewport (root_node_, viewport);

    create_camera ();
    create_lighting ();

    //RMcolor4D specular = {0.4F, 0.4F, 0.4F, 1.0F};
    //object_node_ = rmNodeNew ("Object", RM_RENDERPASS_3D, RM_RENDERPASS_OPAQUE);
    object_node_ = rmNodeNew ("Object", RM_RENDERPASS_ALL, RM_RENDERPASS_ALL);
    rmNodeSetRotateMatrix (object_node_,  &initial_transform);
    //rmNodeSetSpecularColor    (object_node_, &specular);
    //rmNodeSetSpecularExponent (object_node_, 20);//*/  
    rmNodeAddChild (scene_node_, object_node_);    //fix remove
}

void QRMWidget::
fit_center ()
{
    INVARIANT;

    rmNodeComputeBoundingBox           (object_node_);
    //rmNodeComputeCenterFromBoundingBox (object_node_);

    RMvertex3D  center;
    rmNodeGetCenter (object_node_, &center);
    rmNodeSetCenter (camera_node_, &center);

    MM::to_user().status("fit_center");
    updateGL ();

    INVARIANT;
}

void QRMWidget::
fit_to_screen ()
{
    INVARIANT;
    //return; //fix

    RMcamera3D *camera;
    rmNodeGetSceneCamera3D (camera_node_, &camera);

    RMvertex3D  center;
    double x_center, y_center, z_center, scale;
    model_view_->get_dimensions (x_center, y_center, z_center, scale);

    if (scale == 0)
        return;

    //rmNodeUnionAllBoxes (root_node_);
    //rmNodeComputeCenterFromBoundingBox(root_node_);
    //rmNodeComputeBoundingBox(root_node_);

    double new_eye_distance = (scale/2)
                        / tan (rmCamera3DGetFOV (camera) * 3.14 / 180 / 2);

    set_eye_distance (new_eye_distance);
    
    center.x = -x_center;
    center.y = -y_center;
    center.z = -z_center;
    rmNodeSetTranslateVector (camera_node_, &center);

    center.x = x_center;
    center.y = y_center;
    center.z = z_center;
    rmNodeSetCenter          (camera_node_, &center);

    rmCamera3DDelete (camera);
    //MM::to_user().status ("Fit to screen.");
    updateGL ();

    INVARIANT;
}

void QRMWidget::
flush_camera ()
{
    RMmatrix transform;
    rmMatrixIdentity      (&transform);
    rmNodeSetRotateMatrix (camera_node_, &transform);
    updateGL ();
}

void QRMWidget::
demo_rotation (int steps)
{
    for (int i=0;  i<steps;  ++i)
    {
         double x = 400. + 300. * cos(.02 * i + 3.14);
         double y = 300. + 200. * sin(.02 * i);

         QMouseEvent event
            (QEvent::MouseMove, QPoint(int(x), int(y)), 
	     Qt::NoButton, Qt::LeftButton);
         QApplication::sendEvent (this, &event);
    }
}

double QRMWidget::
eye_distance () const
{
    INVARIANT;

    RMcamera3D *camera;
    rmNodeGetSceneCamera3D (camera_node_, &camera);
    
    double result = eye_distance (camera);
    
    rmCamera3DDelete (camera);

    INVARIANT;
    return result;
}

double QRMWidget::
eye_distance (RMcamera3D *camera) const
{
    INVARIANT;

    RMvertex3D  at_point, eye_point, eye_vector;

    rmCamera3DGetEye (camera, &eye_point);
    rmCamera3DGetAt  (camera, &at_point);
    rmVertex3DDiff   (&eye_point, &at_point, &eye_vector);

    INVARIANT;
    return rmVertex3DMag (&eye_vector);
}

void QRMWidget::
set_eye_distance (double distance)
{
    INVARIANT;

    RMcamera3D *camera;
    rmNodeGetSceneCamera3D (camera_node_, &camera);
    set_eye_distance (camera, distance);

    rmCamera3DSetYon (camera, 10000.F);//fix

    rmNodeSetSceneCamera3D (camera_node_, camera);
    rmCamera3DDelete (camera);

    INVARIANT;
}

void QRMWidget::
set_eye_distance (RMcamera3D *camera, double distance)
{
    INVARIANT;

    RMvertex3D  at_point, old_eye_point, new_eye_point, eye_vector;

    rmCamera3DGetEye    (camera, &old_eye_point);
    rmCamera3DGetAt     (camera, &at_point);
    rmVertex3DDiff      (&old_eye_point, &at_point, &eye_vector);
    rmVertex3DNormalize (&eye_vector);
    eye_vector.x *= distance;
    eye_vector.y *= distance;
    eye_vector.z *= distance;
    rmVertex3DSum (&at_point, &eye_vector, &new_eye_point);
    rmCamera3DSetEye (camera, &new_eye_point);

    if (model_view_ != 0)                //fix
    {
        int lod = int(50/sqrt(distance));
        //MM::to_user().status (lod);
        model_view_->set_level_of_details (lod);
    }
    INVARIANT;
}

void QRMWidget::
paintGL ()
{
    //PP
    INVARIANT;

/*    RMenum result = rmPipeMakeCurrent (pipe_);
    if (result != RM_CHILL)
        FLAW ("rmPipeMakeCurrent");//*/

    MM::Timer timer;
    QTime t;
    t.start();
    RMtime *time_1 = rmTimeNew ();
    RMtime *time_2 = rmTimeNew ();
    rmTimeCurrent (time_1);

    rmFrame (pipe_, root_node_);

/*    rmTimeCurrent (time_2);
    MM::to_user().status 
        (MM::Text(rmTimeDifferenceMS (time_1, time_2)) + " Milliseconds           " 
        + t.restart() + " ms (QTime)             "
        + timer.elapsed() + " s (MM::Timer)");
//*/
    rmTimeDelete (time_1);
    rmTimeDelete (time_2);

/*    result = rmPipeClose  (pipe_);
    if (result != RM_CHILL)
        FLAW ("rmPipeClose");//*/
    
    //rmPipeSetDisplayListEnable (pipe_, RM_TRUE);
    INVARIANT;
    //PP
}

void QRMWidget::
create_lighting ()
{
    //rmDefaultLighting (root_); return;

    //RMcolor4D background_color = {.30f, .22f, .15f, 1.0f};
    //RMcolor4D background_color = {.30f, .26f, .20f, 1.0f};
    //RMcolor4D background_color = {.30F, .28F, .25F, 1.0F};

    //RMcolor4D background_color = {.35F, .33F, .30F, 1.0F};
    //RMcolor4D background_color = {.35F, .33F, .28F, 1.0F};
    //RMcolor4D background_color = {60.F/256.F, 90.F/256.F, 80.F/256.F, 1.0F};
    //RMcolor4D background_color = {80.F/256.F, 70.F/256.F, 60.F/256.F, 1.0F};

    double k = 1.2;
    //RMcolor4D background_color = {k*95.F/256.F, k*85.F/256.F, k*70.F/256.F, 1.0F};
    RMcolor4D background_color = {k*95.F/256.F, k*88.F/256.F, k*75.F/256.F, 1.0F};

    //RMcolor4D background_color = {242.F/256.F, 242.F/256.F, 232.F/256.F, 1.0F};
    //RMcolor4D background_color = {223.F/256.F, 223.F/256.F, 211.F/256.F, 1.0F};
//    RMcolor4D background_color = {199.F/256.F, 199.F/256.F, 183.F/256.F, 1.0F};
    //RMcolor4D background_color = {190.F/256.F, 190.F/256.F, 175.F/256.F, 1.0F};
    
    rmNodeSetSceneBackgroundColor (root_node_, &background_color);

    //RMcolor4D unlit_color = {0.9f, 0.9f, 0.9f, 1.0f};
    RMcolor4D unlit_color = {0.5f, 0.5f, 0.5f, 1.f};
    rmNodeSetUnlitColor (root_node_, &unlit_color);

    //RMcolor4D  ambient_0 = {0.2f, 0.2f, 0.2f, 1.0f};
    RMcolor4D  ambient_0 = {0.25f, 0.25f, 0.25f, 1.0f};
    //RMcolor4D  diffuse_0 = {0.7f, 0.7f, 0.7f, 1.0f};
    RMcolor4D  diffuse_0 = {0.60f, 0.60f, 0.60f, 1.0f};
    RMcolor4D  diffuse_1 = {0.1f, 0.1f, 0.1f, 1.0f};

    //RMcolor4D  specular_0 = {0.2f, 0.2f, 0.2f, 1.0f};
    RMcolor4D  specular_0 = {0.3f, 0.3f, 0.3f, 1.0f};
    //RMcolor4D  specular_0 = {0.4f, 0.4f, 0.4f, 1.0f};
    //RMcolor4D  specular_0 = {0.7f, 0.7f, 0.7f, 1.0f};
    //RMcolor4D  specular_0 = {1.f, 1.f, 1.f, 1.0f};    
    RMcolor4D  specular_1 = {0.7f, 0.7f, 0.7f, 1.0f};

    RMvertex3D position_0 = {0.0,  0.0,  1.0}; 
    RMvertex3D position_1 = {-5.0, 5.0,  5.0};

    RMlight *light_0 = rmLightNew();
    if (light_0 == 0)
        FLAW ("Can not create first light source");;

    rmLightSetType  (light_0, RM_LIGHT_DIRECTIONAL); 
    rmLightSetColor (light_0, &ambient_0, &diffuse_0, &specular_0); 
    rmLightSetXYZ   (light_0, &position_0);

    RMlight *light_1 = rmLightNew();
    if (light_1 == 0)
        FLAW ("Can not create second light source");;
    
    rmLightSetType  (light_1, RM_LIGHT_DIRECTIONAL); 
    rmLightSetColor (light_1, NULL, &diffuse_1, &specular_1);
    rmLightSetXYZ   (light_1, &position_1);

    rmNodeSetSceneLight (root_node_, RM_LIGHT0, light_0);
    //rmNodeSetSceneLight (root_, RM_LIGHT1, light_1);   

    rmNodeSetSpecularExponent (root_node_, 20);

     
    rmLightDelete (light_1);
    rmLightDelete (light_0); 
}

void QRMWidget::
create_camera ()
{
    RMcamera3D *camera;
    RMvertex3D eye = {0.0, 0.0, 1000.0};
    RMvertex3D up  = {0.0, 1.0, 0.0};
    RMvertex3D at  = {0.0, 0.0, 0.0} ;
    float hither   = 0.1F, yon = 500.F;

    camera = rmCamera3DNew();
    
    rmCamera3DSetEye        (camera, &eye);
    rmCamera3DSetAt         (camera, &at);
    rmCamera3DSetUpVector   (camera, &up);
    rmCamera3DSetHither     (camera, hither);
    rmCamera3DSetYon        (camera, yon);
    rmCamera3DSetAspectRatio(camera, 1.0F);
    rmCamera3DSetProjection (camera, RM_PROJECTION_PERSPECTIVE); 
    //rmCamera3DSetFOV(camera, 40.0F);
    //rmCamera3DSetFOV(camera, 90.0F);
    rmCamera3DSetFOV (camera, 10.0F);//*/

    //rmDefaultCamera3D(camera);
    //rmCamera3DComputeViewFromGeometry (camera, root_node_, width(), height());
    /*//if (stereo_format != RM_MONO_CHANNEL)
    {
        rmCamera3DSetStereo(camera,RM_TRUE);
        rmCamera3DSetEyeSeparation(camera,2.5F);
        rmCamera3DSetFocalDistance (camera,0.707F);
    }//*/

    {
       float vp[4] = {0.0, 0.0, 1.0, 1.0};
       rmCamera3DResetAspectRatio (camera, vp, width(), height());
    }//*/

    //rmCamera3DComputeViewFromGeometry(camera,root_,width(),height());

    rmNodeSetSceneCamera3D (camera_node_, camera);
    //rmNodeSetSceneCamera3D (scene_, camera);
    rmCamera3DDelete (camera);
}

void QRMWidget::
resizeGL (int width, int height)
{
    RMcamera3D *camera = 0;

    if (rmNodeGetSceneCamera3D (camera_node_, &camera) != RM_WHACKED)
    {
        rmCamera3DSetAspectRatio (camera, (float(width) / float(height)));
        rmNodeSetSceneCamera3D   (camera_node_, camera);
    }

    if (camera == 0)
    {
        QMessageBox::warning (0 , "", "the cameraNode does not contain "
                             "either a 2D or 3D camera scene parameter!");
    }

    rmPipeSetWindowSize (pipe_, width, height);

    QGLWidget::resizeGL (width, height);
}

void QRMWidget::
mousePressEvent (QMouseEvent * event)
{
    INVARIANT;

    last_position_ = event->pos();

    QGLWidget::mousePressEvent (event);

    INVARIANT;
}

void QRMWidget::
mouseMoveEvent (QMouseEvent * event)
{
    INVARIANT;

    //float hypotenuse = sqrt (2);
    int scale = min (width(), height());
    
    float last_x    =  pixel_to_float (last_position_.x(), width (), scale);
    float last_y    = -pixel_to_float (last_position_.y(), height(), scale);
    float current_x =  pixel_to_float (event->x(),         width (), scale);
    float current_y = -pixel_to_float (event->y(),         height(), scale);//*/

    //double dx = current_x - last_x;
    double dy = current_y - last_y;

    if (event->state () & (Qt::LeftButton))  //fix
    {
        if (event->state () & Qt::ShiftModifier ) // Zoom
        {
            double increment = 1. + mouse_scaling_increment * dy;
            double distance = eye_distance ();
            set_eye_distance (distance * increment);
        }

        else if (event->state () & Qt::ControlModifier ) //Move
        {
            RMcamera3D *camera;
            rmNodeGetSceneCamera3D (camera_node_, &camera);
            //rmauxTranslate (camera, &last_x_, &last_y_, &current_x, &current_y);
            //rmNodeSetSceneCamera3D (root_, camera);

            RMvertex3D  at_point, eye_point, up_point, 
                        eye_vector, up_vector, rigth_vector, movement;

            rmCamera3DGetEye      (camera, &eye_point);
            rmCamera3DGetAt       (camera, &at_point);
            rmCamera3DGetUpVector (camera, &up_point);

            rmVertex3DDiff  (&eye_point, &at_point, &eye_vector);
            rmVertex3DDiff  (&up_point,  &at_point, &up_vector);
//            rmVertex3DCross (&up_vector, &eye_vector, &rigth_vector);
            rmVertex3DCross (&eye_vector, &up_vector, &rigth_vector);

            rmVertex3DNormalize (&up_vector);
            rmVertex3DNormalize (&rigth_vector);

            double eye_distance = rmVertex3DMag (&eye_vector);
            rmVertex3DNormalize (&eye_vector);


            double tg_FOV2      = tan ((rmCamera3DGetFOV (camera)*3.14159265/180) / 2);
            double x_movement   = (event->x() - last_position_.x()) 
                                    * eye_distance * 2 * tg_FOV2 / height ();

            double y_movement   = (event->y() - last_position_.y()) 
                                    * eye_distance * 2 * tg_FOV2 / height();

            RMvertex3D v;
            rmVertex3DCross (&rigth_vector, &eye_vector, &v);
            rmVertex3DNormalize (&v);

            v.x *= y_movement;
            v.y *= y_movement;
            v.z *= y_movement;

            rigth_vector.x *= x_movement;
            rigth_vector.y *= x_movement;
            rigth_vector.z *= x_movement;

            rmVertex3DSum (&v, &rigth_vector, &movement);

            ///rmVertex3DSum (&eye_point, &movement, &eye_point);
            ///rmVertex3DSum (&at_point,  &movement, &at_point);
            //rmVertex3DSum (&up_point,  &movement, &up_point);

            ///rmCamera3DSetEye      (camera, &eye_point);
            ///rmCamera3DSetAt       (camera, &at_point);
            //rmCamera3DSetUpVector (camera, &up_point);

            //rmNodeSetSceneCamera3D (root_, camera);//*/
            
            RMvertex3D  last_movement; 
            rmNodeGetTranslateVector (camera_node_, &last_movement);
            rmVertex3DDiff  (&last_movement, &movement, &movement);
            rmNodeSetTranslateVector (camera_node_, &movement);//*/

            rmCamera3DDelete (camera);
        }

        else // Rotate
        {
            RMmatrix last_transform;
            RMmatrix result_transform;

            float k = .6f;             //fix magic
            last_x    *= k;
            last_y    *= k;
            current_x *= k;
            current_y *= k;//*/

            rmauxArcBall          (&last_x, &last_y, &current_x, &current_y,
                                   &result_transform);
            rmNodeGetRotateMatrix (camera_node_, &last_transform);
            rmMatrixMultiply      (&last_transform, &result_transform, 
                                   &result_transform);
            rmNodeSetRotateMatrix (camera_node_, &result_transform);
        }
        updateGL ();
    }
    
    last_position_ = event->pos();
    //QGLWidget::mouseMoveEvent (event);
    INVARIANT;
}

void QRMWidget::
closeEvent (QCloseEvent * e)
{
    INVARIANT;
    //main_window_.remove_render_area (identifier_);
//    MM::log () << " QRMWidget::closeEvent\n";
    QGLWidget::closeEvent (e);
    INVARIANT;
}

/*bool QRMWidget::close ()
{
//    MM::log () << " QRMWidget::close\n";
    return QWidget::close ();
}//*/

void QRMWidget::
windowActivationChange (bool oldActive)
{
    QGLWidget::windowActivationChange (oldActive);

/*    if (isActiveWindow())
    {
        model_view_->set_focus ();                      //fix remove
        //RMenum result = rmPipeMakeCurrent (pipe_);
        //if (result != RM_CHILL)
        //    FLAW ("rmPipeMakeCurrent");
    }
    else
    {
        MM::log() << 111111;
        RMenum result = rmPipeClose  (pipe_);
        if (result != RM_CHILL)
            FLAW ("rmPipeClose");
        MM::log() << 222222;
    }//*/
}

void QRMWidget::
timerEvent (QTimerEvent *e)
{
    if (e->timerId() == idle_timer_id_)
    {
        //#ifdef WIN32
        //SetPriorityClass (GetCurrentProcess(), NORMAL_PRIORITY_CLASS);
        //#endif

        //MM::sleep (100);
    }
    else
    {
        QGLWidget::timerEvent (e);
    }
}
