// 
// This file is part of MolMeccano, a cross-platform C++ chemical library.
//
// Copyright (C) 2001-2007 Alexey Nikitin
//

#include "Path.h"
#include "Create.h"
#include "Application.h"
#include "Configuration.h"
#include "Dynamic_configuration.h"

#ifdef MM_WIN32
#include <direct.h>
#else
#include <unistd.h>
#endif

#ifdef MM_WIN32
#pragma warning (disable : 4996)
#endif

#include "Log.h"

#include <string>
#include <iostream>


#include <QApplication>
#include <QSettings>
//#include <QMainWindow>
//#include "Graphic_configuration.h"

using namespace MM;

int main (int argc, char* argv[])
{
    using namespace std;

    getcwd (Path::start_dir(), 1000);
    cout   << "Start at " << Path::start_dir () << "\n\n";
    log () << "Start at " << Text(Path::start_dir ()) << "\n\n";

    QApplication    application (argc, argv);

    QSettings settings("AgileMolecule", argv[0]);

    if (argc == 2)
    {
        if (QString (argv[1]) == "install")
        {
            settings.setValue("bin", Path::start_dir());

            string   message;
            message += "Installed in ";
            message += Path::start_dir ();
            message += "\n";
            cout  << message;
            log() << message.c_str();
            return 0;
        }
        else if (QString (argv[1]) == "uninstall")
        {
            settings.clear ();

            string   message;
            message += "Uninstalled ";
            message += Path::start_dir ();
            message += "\n";
            cout  << message;
            log() << message.c_str();
            return 0;
        }
    }

    QString bin = settings.value ("bin").toString ();
    
    if (!bin.isEmpty ())
    {
        if (!bin.endsWith("bin"))
        {
            string   message;
            message += "Program should be installed in dir /bin\nbut it was installed in ";
            message += bin.ascii();
            cout  << message;
            log() << message.c_str();
            return 1;
        }

        QString prefix = bin.left (bin.length() - 3);
        Path::home() = prefix.ascii();
    }


    Dynamic_configuration config (argc, argv);

//    Graphic_configuration config (argc, argv);
//    new QApplication (argc, argv);

//    QApplication app(argc, argv);
//    QMainWindow *w = new QMainWindow;
//    w->show ();
//    return qApp->exec();

//    QMainWindow & w = singleton <Application>();
//    w.show();
    

    int result = singleton <Application> ().run();
    Prototype_repository::destroy();
    return result;
}

//*/
/*
#include "Create.h"
#include "Path.h"
#include "Model.h"
#include "Force_field.h"
#include "CC_energy.h"
#include <fstream>
#include "Configuration.h"
#include "Dynamic_configuration.h"

using namespace std;
using namespace MM;

int main (int argc, char* argv[])
{
    Dynamic_configuration config (argc, argv);

    ofstream out ((Path::report () + "result.txt").c_str());

    Text bonds             = Path::force_field() + "a94str.txt";
    Text VdW               = Path::force_field() + "a94nbd.txt";
    Text angles            = Path::force_field() + "a94ben.txt";
    Text torsions          = Path::force_field() + "a94tor.txt";
    Text improper_torsions = Path::force_field() + "a94imp.txt";

    Force_field     *amber94 = new Force_field  
        (bonds.c_str(), VdW.c_str(), angles.c_str(), 
         torsions.c_str(), improper_torsions.c_str(), 
         1/1.2, .5, 1.);

    Model *model =  prototype <Model> (). clone ();
    model->load (Path::tests() + "B_DNA_6.hin");
    Model_energy energy (*model, *model, *amber94);
    out << "B_DNA_6.hin                             Energy = " 
         << energy.energy () << endl;
    model->save_as ("B_DNA_6.mol");


    model->load ("model-I-A-3A_600K-500ps-2-free-turn.hin");
    Model_energy energy2 (*model, *model, *amber94);
    out << "model-I-A-3A_600K-500ps-2-free-turn.hin Energy = " 
         << energy2.energy () << endl;
    

    return 0;
}

// result.txt
//
//B_DNA_6.hin                             Energy = 5177.48
//model-I-A-3A_600K-500ps-2-free-turn.hin Energy = 1.35802

//*/

/*
#include "mm.md.h"

using namespace MM;

int main (int argc, char* argv[])
{
    Dynamic_configuration config (argc, argv);

    //======================================================

    MD_simulator md;
    md.model().load (Path::tests() + "Harmonic_H2.xyz");

    Harmonic_potential *interaction 
        = new Harmonic_potential (md.model().atom(0), md.model().atom(1));
    interaction->set_equilibrium_distance (6.);
    interaction->set_force_constant       (.25);
    md.adopt_interaction (interaction);

    int steps = 20;
    md.set_steps (steps);
    md.set_step_length ((1./steps) * 2. * __Pi__);
    md.after_each_step ("Collect_phase_trajectory"); 
    md.run ();

    //======================================================

    return 0;
}

//*/
