/* 
    This file is part of the ChemCon molecular mechanics 
    and molecular graphics program.

    Copyright (C) 1997-2002  Alexei Nikitin 

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "Painter_2D.h"

#include "Image.h"
#include <memory.h>

//#define abs( x ) ( x < 0 ? -x : x )
//#include <stdlib.h>
//#include <math.h>

Painter_2D::
Painter_2D( Image & image )
:
    image_( image ),
    //pixel_  ( ( Rgb * ) image.pixels() ), //fix Rgb to unsigned char *
    width_  ( image.width () ),
    height_ ( image.height() ),
    X_      ( 0 ),
    Y_      ( 0 ),
    foreground_color_( rgb( 255, 255, 255 ) ),
    background_color_( rgb( 0, 0, 0 ) ),
    //pitch_(  4 * img.width() )
    corrected_height_( image.height() - 1 )
{
}

Painter_2D::~Painter_2D()
{
    //pixel_  = 0;
    width_  = 0;
    height_ = 0;
    X_      = 0;
    Y_      = 0;
    foreground_color_ = rgb( 0, 0, 0 );
    background_color_ = rgb( 0, 0, 0 );
    //    pitch_  = 0;
    corrected_height_ = 0;
}


void  Painter_2D::
line( int X0, int Y0, int X1, int Y1 )
{
    unsigned long ErrorAcc, ErrorAdj;
    int DeltaX, DeltaY, XDir, Temp;

    // Make sure the line runs top to bottom
    if( Y0 > Y1 )
    {
        Temp = Y0;
        Y0 = Y1;
        Y1 = Temp;
        Temp = X0;
        X0 = X1;
        X1 = Temp;
    }

    point( X0, Y0 ); // draw the initial pixel

    if( ( DeltaX = X1 - X0 ) >= 0 )
    {
        XDir = 1;
    }
    else
    {
        XDir = -1;
        DeltaX = -DeltaX; // make DeltaX positive
    }

    if( ( DeltaY = Y1 - Y0 ) == 0 )  // done if only one point in the line
           if( DeltaX == 0 ) return;

    ErrorAcc = 0x8000; // initialize line error accumulator to .5, so we can
    // advance when we get halfway to the next pixel

    // Is this an X-major or Y-major line?
    if( DeltaY > DeltaX )
    {
        // Y-major line; calculate 16-bit fixed-point fractional part of a
        // pixel that X advances each time Y advances 1 pixel
        ErrorAdj = ( ( ( ( unsigned long ) DeltaX << 17 ) /
           ( unsigned long ) DeltaY ) + 1 ) >> 1;

        // Draw all pixels between the first and last
        do
        {
            ErrorAcc += ErrorAdj; // calculate error for this pixel
            if ( ErrorAcc & ~0xFFFFL )
            {
                // The error accumulator turned over, so advance the X coord
                X0 += XDir;
                ErrorAcc &= 0xFFFFL; // clear integer part of result
            }

            Y0++; // Y-major, so always advance Y
            point( X0, Y0 );

        }
        while( --DeltaY );
        return;
    }
    // It's an X-major line; calculate 16-bit fixed-point fractional part of a
    // pixel that Y advances each time X advances 1 pixel
    ErrorAdj = ( ( ( ( unsigned long ) DeltaY << 17 ) /
       ( unsigned long ) DeltaX ) + 1 ) >> 1;
    // Draw all remaining pixels
    do
    {
        ErrorAcc += ErrorAdj; // calculate error for this pixel
        if( ErrorAcc & ~0xFFFFL )
        {
            // The error accumulator turned over, so advance the Y coord
            Y0++;
            ErrorAcc &= 0xFFFFL; // clear integer part of result
        }
        X0 += XDir; // X-major, so always advance X
        point( X0, Y0 );
    }
    while( --DeltaX );
}

void  Painter_2D::
Wu_line( int X0, int Y0, int X1, int Y1 )
{
    //    unsigned int  IntensityShift, ErrorAdj, ErrorAcc;
    //    unsigned int  ErrorAccTemp, Weighting, WeightingComplementMask;
    //    int            DeltaX, DeltaY, Temp, XDir;
    unsigned short  IntensityShift, ErrorAdj, ErrorAcc;
    unsigned short  ErrorAccTemp, Weighting, WeightingComplementMask;
    short           DeltaX, DeltaY, Temp, XDir;
    Byte            R    = red   ( foreground_color_ );
    Byte            G    = green ( foreground_color_ );
    Byte            B    = blue  ( foreground_color_ );
    Byte            R_bg = red   ( background_color_ );
    Byte            G_bg = green ( background_color_ );
    Byte            B_bg = blue  ( background_color_ );

    // Make sure the line runs top to bottom
    if( Y0 > Y1 )
    {
        Temp = ( short )Y0;
        Y0 = Y1;
        Y1 = ( int )Temp;
        Temp = ( short )X0;
        X0 = X1;
        X1 = Temp;
    }
    // Draw the initial pixel, which is always exactly intersected by
    // the line and so needs no weighting
    point( X0, Y0 );

    if( ( DeltaX = X1 - X0 ) >= 0 )
    {
        XDir = 1;
    }
    else
    {
        XDir = -1;
        DeltaX = -DeltaX; // make DeltaX positive
    }
    // Special-case horizontal, vertical, and diagonal lines, which
    // require no weighting because they go right through the center of
    // every pixel
    if( ( DeltaY = Y1 - Y0 ) == 0 )
    {
        // Horizontal line
        while ( DeltaX-- != 0 )
        {
            X0 += XDir;
            point( X0, Y0 );
        }
        return;
    }

    if( DeltaX == 0 )
    {
        // Vertical line
        do
        {
            Y0++;
            point( X0, Y0 );
        }
        while ( --DeltaY != 0 );
        return;
    }

    if( DeltaX == DeltaY )
    {
        // Diagonal line
        do
        {
            X0 += XDir;
            Y0++;
            point( X0, Y0 );
        }
        while ( --DeltaY != 0 );
        return;
    }

    // Line is not horizontal, diagonal, or vertical

    ErrorAcc = 0; // initialize the line error accumulator to 0

    // # of bits by which to shift ErrorAcc to get intensity level
    //IntensityShift = 16 - IntensityBits;
    IntensityShift = 16 - 8;

    // Mask used to flip all bits in an intensity weighting, producing the
    // result (1 - intensity weighting)
    //WeightingComplementMask = NumLevels - 1;
    WeightingComplementMask = 256 - 1;

    // Is this an X-major or Y-major line?
    if ( DeltaY > DeltaX )
    {
        // Y-major line; calculate 16-bit fixed-point fractional part of a
        //pixel that X advances each time Y advances 1 pixel, truncating the
        //result so that we won't overrun the endpoint along the X axis
        ErrorAdj = ( ( unsigned short ) DeltaX << 16 ) /
           ( unsigned short ) DeltaY;

        // Draw all pixels other than the first and last
        while( --DeltaY )
        {
            ErrorAccTemp = ErrorAcc; // remember currrent accumulated error
            ErrorAcc += ErrorAdj; // calculate error for next pixel
            if( ErrorAcc <= ErrorAccTemp )
            {
                // The error accumulator turned over, so advance the X coord
                X0 += XDir;
            }
            Y0++; // Y-major, so always advance Y

            // The IntensityBits most significant bits of ErrorAcc give us the
            // intensity weighting for this pixel, and the complement of the
            // weighting for the paired pixel
            Weighting = ErrorAcc >> IntensityShift;

            unsigned short
            Shift  = ( 255 - us_abs( Weighting  -
               ( Weighting ^ WeightingComplementMask ) ) ) >> 2;

            point( X0 + XDir, Y0,
            ( R * ( Weighting + Shift )
            + R_bg * ( ( Weighting ^ WeightingComplementMask ) - Shift ) ) >> 8,
            ( G * ( Weighting + Shift )
            + G_bg * ( ( Weighting ^ WeightingComplementMask ) - Shift ) ) >> 8,
            ( B * ( Weighting + Shift )
            + B_bg * ( ( Weighting ^ WeightingComplementMask ) -
               Shift ) ) >> 8 );
            point( X0, Y0,
            ( R * ( ( Weighting ^ WeightingComplementMask ) + Shift )
            + R_bg * ( Weighting - Shift ) ) >> 8,
            ( G * ( ( Weighting ^ WeightingComplementMask ) + Shift )
            + G_bg * ( Weighting - Shift ) ) >> 8,
            ( B * ( ( Weighting ^ WeightingComplementMask ) + Shift )
            + B_bg * ( Weighting - Shift ) ) >> 8 );
        }
        // Draw the final pixel, which is always exactly intersected by the line
        // and so needs no weighting
        point( X1, Y1 );
    }
    else
    {
        // It's an X-major line; calculate 16-bit fixed-point fractional part of a
        // pixel that Y advances each time X advances 1 pixel, truncating the
        // result to avoid overrunning the endpoint along the X axis
        ErrorAdj = ( ( unsigned short ) DeltaY << 16 ) /
           ( unsigned short ) DeltaX;

        // Draw all pixels other than the first and last
        while( --DeltaX )
        {
            ErrorAccTemp = ErrorAcc; // remember currrent accumulated error
            ErrorAcc += ErrorAdj; // calculate error for next pixel
            if( ErrorAcc <= ErrorAccTemp )
            {
                // The error accumulator turned over, so advance the Y coord
                Y0++;
            }
            X0 += XDir; // X-major, so always advance X
            // The IntensityBits most significant bits of ErrorAcc give us the
            // intensity weighting for this pixel, and the complement of the
            // weighting for the paired pixel
            Weighting = ErrorAcc >> IntensityShift;
            //BaseColor + (Weighting ^ WeightingComplementMask));

            unsigned short
            Shift = ( 255 - us_abs( Weighting  -
               ( Weighting ^ WeightingComplementMask ) ) ) >> 2;
            //Shift = 0;

            point( X0, Y0 + 1,
            ( R * ( Weighting + Shift )
            + R_bg * ( ( Weighting ^ WeightingComplementMask ) - Shift ) ) >> 8,
            ( G * ( Weighting + Shift )
            + G_bg * ( ( Weighting ^ WeightingComplementMask ) - Shift ) ) >> 8,
            ( B * ( Weighting + Shift )
            + B_bg * ( ( Weighting ^ WeightingComplementMask ) -
               Shift ) ) >> 8 );
            point( X0, Y0,
            ( R * ( ( Weighting ^ WeightingComplementMask ) + Shift )
            + R_bg * ( Weighting - Shift ) ) >> 8,
            ( G * ( ( Weighting ^ WeightingComplementMask ) + Shift )
            + G_bg * ( Weighting - Shift ) ) >> 8,
            ( B * ( ( Weighting ^ WeightingComplementMask ) + Shift )
            + B_bg * ( Weighting - Shift ) ) >> 8 );
        }
        // Draw the final pixel, which is always exactly intersected by the line
        //   and so needs no weighting
        point( X1, Y1 );
    }
}

void  Painter_2D::
Wu_line2( int X0, int Y0, int X1, int Y1 )
{
    //    unsigned int  IntensityShift, ErrorAdj, ErrorAcc;
    //    unsigned int  ErrorAccTemp, Weighting, WeightingComplementMask;
    //    int            DeltaX, DeltaY, Temp, XDir;
    unsigned short  IntensityShift, ErrorAdj, ErrorAcc;
    unsigned short  ErrorAccTemp, Weighting, WeightingComplementMask;
    short            DeltaX, DeltaY, Temp, XDir;
    Byte            R    = red   ( foreground_color_ );
    Byte            G    = green ( foreground_color_ );
    Byte            B    = blue  ( foreground_color_ );
    Byte            R_bg = red   ( background_color_ );
    Byte            G_bg = green ( background_color_ );
    Byte            B_bg = blue  ( background_color_ );

    // Make sure the line runs top to bottom
    if( Y0 > Y1 )
    {
        Temp = Y0;
        Y0 = Y1;
        Y1 = Temp;
        Temp = X0;
        X0 = X1;
        X1 = Temp;
    }
    // Draw the initial pixel, which is always exactly intersected by
    // the line and so needs no weighting
    point( X0, Y0 );

    if( ( DeltaX = X1 - X0 ) >= 0 )
    {
        XDir = 1;
    }
    else
    {
        XDir = -1;
        DeltaX = -DeltaX; // make DeltaX positive
    }
    // Special-case horizontal, vertical, and diagonal lines, which
    // require no weighting because they go right through the center of
    // every pixel
    if( ( DeltaY = Y1 - Y0 ) == 0 )
    {
        // Horizontal line
        while ( DeltaX-- != 0 )
        {
            X0 += XDir;
            point( X0, Y0 );
            point( X0, Y0 + 1 );
        }
        return;
    }

    if( DeltaX == 0 )
    {
        // Vertical line
        do
        {
            Y0++;
            point( X0, Y0 );
            point( X0 + 1, Y0 );
        }
        while ( --DeltaY != 0 );
        return;
    }

    if( DeltaX == DeltaY )
    {
        // Diagonal line
        do
        {
            X0 += XDir;
            Y0++;
            point( X0, Y0 );
            point( X0 - 1, Y0 );
        }
        while ( --DeltaY != 0 );
        return;
    }

    // Line is not horizontal, diagonal, or vertical

    ErrorAcc = 0; // initialize the line error accumulator to 0

    // # of bits by which to shift ErrorAcc to get intensity level
    //IntensityShift = 16 - IntensityBits;
    IntensityShift = 16 - 8;

    // Mask used to flip all bits in an intensity weighting, producing the
    // result (1 - intensity weighting)
    //WeightingComplementMask = NumLevels - 1;
    WeightingComplementMask = 256 - 1;

    // Is this an X-major or Y-major line?
    if ( DeltaY > DeltaX )
    {
        // Y-major line; calculate 16-bit fixed-point fractional part of a
        //pixel that X advances each time Y advances 1 pixel, truncating the
        //result so that we won't overrun the endpoint along the X axis
        ErrorAdj = ( ( unsigned short ) DeltaX << 16 ) /
           ( unsigned short ) DeltaY;

        // Draw all pixels other than the first and last
        while( --DeltaY )
        {
            ErrorAccTemp = ErrorAcc; // remember currrent accumulated error
            ErrorAcc += ErrorAdj; // calculate error for next pixel
            if( ErrorAcc <= ErrorAccTemp )
            {
                // The error accumulator turned over, so advance the X coord
                X0 += XDir;
            }
            Y0++; // Y-major, so always advance Y

            // The IntensityBits most significant bits of ErrorAcc give us the
            // intensity weighting for this pixel, and the complement of the
            // weighting for the paired pixel
            Weighting = ErrorAcc >> IntensityShift;

            unsigned short
            Shift  = ( 255 - us_abs( Weighting  -
               ( Weighting ^ WeightingComplementMask ) ) ) >> 3;

            point( X0 + XDir, Y0,
            ( R * ( Weighting + Shift )
            + R_bg * ( ( Weighting ^ WeightingComplementMask ) - Shift ) ) >> 8,
            ( G * ( Weighting + Shift )
            + G_bg * ( ( Weighting ^ WeightingComplementMask ) - Shift ) ) >> 8,
            ( B * ( Weighting + Shift )
            + B_bg * ( ( Weighting ^ WeightingComplementMask ) -
               Shift ) ) >> 8 );
            point( X0, Y0, R, G, B );
            point( X0 - XDir, Y0,
            ( R * ( ( Weighting ^ WeightingComplementMask ) + Shift )
            + R_bg * ( Weighting - Shift ) ) >> 8,
            ( G * ( ( Weighting ^ WeightingComplementMask ) + Shift )
            + G_bg * ( Weighting - Shift ) ) >> 8,
            ( B * ( ( Weighting ^ WeightingComplementMask ) + Shift )
            + B_bg * ( Weighting - Shift ) ) >> 8 );
        }
        // Draw the final pixel, which is always exactly intersected by the line
        // and so needs no weighting
        point( X1, Y1 );
    }
    else
    {
        // It's an X-major line; calculate 16-bit fixed-point fractional part of a
        // pixel that Y advances each time X advances 1 pixel, truncating the
        // result to avoid overrunning the endpoint along the X axis
        ErrorAdj = ( ( unsigned short ) DeltaY << 16 ) /
           ( unsigned short ) DeltaX;

        // Draw all pixels other than the first and last
        while( --DeltaX )
        {
            ErrorAccTemp = ErrorAcc; // remember currrent accumulated error
            ErrorAcc += ErrorAdj; // calculate error for next pixel
            if( ErrorAcc <= ErrorAccTemp )
            {
                // The error accumulator turned over, so advance the Y coord
                Y0++;
            }
            X0 += XDir; // X-major, so always advance X
            // The IntensityBits most significant bits of ErrorAcc give us the
            // intensity weighting for this pixel, and the complement of the
            // weighting for the paired pixel
            Weighting = ErrorAcc >> IntensityShift;
            //BaseColor + (Weighting ^ WeightingComplementMask));

            unsigned short
            Shift  = ( 255 - us_abs( Weighting  -
               ( Weighting ^ WeightingComplementMask ) ) ) >> 3;
            point( X0, Y0 + 1,
            ( R * ( Weighting + Shift )
            + R_bg * ( ( Weighting ^ WeightingComplementMask ) - Shift ) ) >> 8,
            ( G * ( Weighting + Shift )
            + G_bg * ( ( Weighting ^ WeightingComplementMask ) - Shift ) ) >> 8,
            ( B * ( Weighting + Shift )
            + B_bg * ( ( Weighting ^ WeightingComplementMask ) -
               Shift ) ) >> 8 );
            point( X0, Y0, R, G, B );
            point( X0, Y0 - 1,
            ( R * ( ( Weighting ^ WeightingComplementMask ) + Shift )
            + R_bg * ( Weighting - Shift ) ) >> 8,
            ( G * ( ( Weighting ^ WeightingComplementMask ) + Shift )
            + G_bg * ( Weighting - Shift ) ) >> 8,
            ( B * ( ( Weighting ^ WeightingComplementMask ) + Shift )
            + B_bg * ( Weighting - Shift ) ) >> 8 );
        }
        // Draw the final pixel, which is always exactly intersected by the line
        //   and so needs no weighting
        point( X1, Y1 );
    }
}

void  Painter_2D::
Wu_line3( int X0, int Y0, int X1, int Y1 )
{
    //    unsigned int  IntensityShift, ErrorAdj, ErrorAcc;
    //    unsigned int  ErrorAccTemp, Weighting, WeightingComplementMask;
    //    int            DeltaX, DeltaY, Temp, XDir;
    unsigned short  IntensityShift, ErrorAdj, ErrorAcc;
    unsigned short  ErrorAccTemp, Weighting, WeightingComplementMask;
    short            DeltaX, DeltaY, Temp, XDir;
    Byte            R    = red   ( foreground_color_ );
    Byte            G    = green ( foreground_color_ );
    Byte            B    = blue  ( foreground_color_ );
    Byte            R_bg = red   ( background_color_ );
    Byte            G_bg = green ( background_color_ );
    Byte            B_bg = blue  ( background_color_ );

    // Make sure the line runs top to bottom
    if( Y0 > Y1 )
    {
        Temp = Y0;
        Y0 = Y1;
        Y1 = Temp;
        Temp = X0;
        X0 = X1;
        X1 = Temp;
    }
    // Draw the initial pixel, which is always exactly intersected by
    // the line and so needs no weighting
    point( X0, Y0 );

    if( ( DeltaX = X1 - X0 ) >= 0 )
    {
        XDir = 1;
    }
    else
    {
        XDir = -1;
        DeltaX = -DeltaX; // make DeltaX positive
    }
    // Special-case horizontal, vertical, and diagonal lines, which
    // require no weighting because they go right through the center of
    // every pixel
    if( ( DeltaY = Y1 - Y0 ) == 0 )
    {
        // Horizontal line
        while ( DeltaX-- != 0 )
        {
            X0 += XDir;
            point( X0, Y0 - 1 );
            point( X0, Y0 );
            point( X0, Y0 + 1 );
        }
        return;
    }

    if( DeltaX == 0 )
    {
        // Vertical line
        do
        {
            Y0++;
            point( X0 - 1, Y0 );
            point( X0, Y0 );
            point( X0, Y0 - 1 );
        }
        while ( --DeltaY != 0 );
        return;
    }

    if( DeltaX == DeltaY )
    {
        // Diagonal line
        do
        {
            X0 += XDir;
            Y0++;
            point( X0 - 1, Y0 );
            point( X0, Y0 );
            point( X0 + 1, Y0 );
        }
        while ( --DeltaY != 0 );
        return;
    }

    // Line is not horizontal, diagonal, or vertical

    ErrorAcc = 0; // initialize the line error accumulator to 0

    // # of bits by which to shift ErrorAcc to get intensity level
    //IntensityShift = 16 - IntensityBits;
    IntensityShift = 16 - 8;

    // Mask used to flip all bits in an intensity weighting, producing the
    // result (1 - intensity weighting)
    //WeightingComplementMask = NumLevels - 1;
    WeightingComplementMask = 256 - 1;

    // Is this an X-major or Y-major line?
    if ( DeltaY > DeltaX )
    {
        // Y-major line; calculate 16-bit fixed-point fractional part of a
        //pixel that X advances each time Y advances 1 pixel, truncating the
        //result so that we won't overrun the endpoint along the X axis
        ErrorAdj = ( ( unsigned short ) DeltaX << 16 ) /
           ( unsigned short ) DeltaY;

        // Draw all pixels other than the first and last
        while( --DeltaY )
        {
            ErrorAccTemp = ErrorAcc; // remember currrent accumulated error
            ErrorAcc += ErrorAdj; // calculate error for next pixel
            if( ErrorAcc <= ErrorAccTemp )
            {
                // The error accumulator turned over, so advance the X coord
                X0 += XDir;
            }
            Y0++; // Y-major, so always advance Y

            // The IntensityBits most significant bits of ErrorAcc give us the
            // intensity weighting for this pixel, and the complement of the
            // weighting for the paired pixel
            Weighting = ErrorAcc >> IntensityShift;

            unsigned short
            Shift  = ( 255 - us_abs( Weighting  -
               ( Weighting ^ WeightingComplementMask ) ) ) >> 3;

            point( X0 + XDir + XDir, Y0,
            ( R * ( Weighting + Shift )
            + R_bg * ( ( Weighting ^ WeightingComplementMask ) - Shift ) ) >> 8,
            ( G * ( Weighting + Shift )
            + G_bg * ( ( Weighting ^ WeightingComplementMask ) - Shift ) ) >> 8,
            ( B * ( Weighting + Shift )
            + B_bg * ( ( Weighting ^ WeightingComplementMask ) -
               Shift ) ) >> 8 );
            point( X0 + XDir, Y0, R, G, B );
            point( X0,        Y0, R, G, B );
            point( X0 - XDir, Y0,
            ( R * ( ( Weighting ^ WeightingComplementMask ) + Shift )
            + R_bg * ( Weighting - Shift ) ) >> 8,
            ( G * ( ( Weighting ^ WeightingComplementMask ) + Shift )
            + G_bg * ( Weighting - Shift ) ) >> 8,
            ( B * ( ( Weighting ^ WeightingComplementMask ) + Shift )
            + B_bg * ( Weighting - Shift ) ) >> 8 );
        }
        // Draw the final pixel, which is always exactly intersected by the line
        // and so needs no weighting
        point( X1, Y1 );
    }
    else
    {
        // It's an X-major line; calculate 16-bit fixed-point fractional part of a
        // pixel that Y advances each time X advances 1 pixel, truncating the
        // result to avoid overrunning the endpoint along the X axis
        ErrorAdj = ( ( unsigned short ) DeltaY << 16 ) /
           ( unsigned short ) DeltaX;

        // Draw all pixels other than the first and last
        while( --DeltaX )
        {
            ErrorAccTemp = ErrorAcc; // remember currrent accumulated error
            ErrorAcc += ErrorAdj; // calculate error for next pixel
            if( ErrorAcc <= ErrorAccTemp )
            {
                // The error accumulator turned over, so advance the Y coord
                Y0++;
            }
            X0 += XDir; // X-major, so always advance X
            // The IntensityBits most significant bits of ErrorAcc give us the
            // intensity weighting for this pixel, and the complement of the
            // weighting for the paired pixel
            Weighting = ErrorAcc >> IntensityShift;
            //BaseColor + (Weighting ^ WeightingComplementMask));

            unsigned short
            Shift  = ( 255 - us_abs( Weighting  -
               ( Weighting ^ WeightingComplementMask ) ) ) >> 3;
            point( X0, Y0 + 2,
            ( R * ( Weighting + Shift )
            + R_bg * ( ( Weighting ^ WeightingComplementMask ) - Shift ) ) >> 8,
            ( G * ( Weighting + Shift )
            + G_bg * ( ( Weighting ^ WeightingComplementMask ) - Shift ) ) >> 8,
            ( B * ( Weighting + Shift )
            + B_bg * ( ( Weighting ^ WeightingComplementMask ) -
               Shift ) ) >> 8 );
            point( X0, Y0 + 1, R, G, B );
            point( X0, Y0,     R, G, B );
            point( X0, Y0 - 1,
            ( R * ( ( Weighting ^ WeightingComplementMask ) + Shift )
            + R_bg * ( Weighting - Shift ) ) >> 8,
            ( G * ( ( Weighting ^ WeightingComplementMask ) + Shift )
            + G_bg * ( Weighting - Shift ) ) >> 8,
            ( B * ( ( Weighting ^ WeightingComplementMask ) + Shift )
            + B_bg * ( Weighting - Shift ) ) >> 8 );
        }
        // Draw the final pixel, which is always exactly intersected by the line
        //   and so needs no weighting
        point( X1, Y1 );
    }
}
//*/

void Painter_2D::
clear()
{
    image_.clear(   red  (background_color_), 
                    green(background_color_), 
                    blue (background_color_) );
    /*int i;
    //int size = width_ * height_;
    Rgb * pointer = pixel_;
    //Rgb * end = pointer + size;

    //for( i=0;  i<size;  i++ )  pixel_[i] = background_color_;     // 0.034
    //for( i=0;  i<size;  i++ )  *pointer++ = background_color_;    // 0.035
    //while( pointer<end )   *pointer++ = background_color_;        // 0.035

    for( i = 0;  i < width_;   i++ )  pixel_[ i ] = background_color_;
    for( i = 1;  i < height_;  i++ )
    {
        pointer = pixel_ + i * width_;
        memcpy( pointer, pixel_, 4 * width_ );
    } // 0.034//*/
}
/*
void Painter_2D::
clear( int left, int right, int top, int bottom )
{
    if( left < 0 || right > width_ || top > height_ || bottom < 0 )
        throw; //fix

    int   i,
    width     = right - left + 1,
    height    = top - bottom + 1;

    Rgb   * pointer0,

#ifdef TOP_TO_BOTTOM_Y
    * pointer  = pixel_ + ( corrected_height_ - top ) * width_ + left,
    * end      = pixel_ + ( corrected_height_ - top ) * width_ + right + 1;
#endif
#ifndef TOP_TO_BOTTOM_Y
    * pointer  = pixel_ + top * width_ + left,
    * end      = pixel_ + top * width_ + right + 1;
#endif

    pointer0 = pointer;

    while( pointer < end )   * pointer++ = background_color_;
    for( i = 1;  i < height;  i++ )
    {
#ifdef TOP_TO_BOTTOM_Y
        pointer = pixel_ + ( ( corrected_height_ - ( top + i ) ) ) *
           width_ + left;
#endif
#ifndef TOP_TO_BOTTOM_Y
        pointer = pixel_ + ( top + i ) * width_ + left;
#endif
        memcpy( pointer, pointer0, 4 * width );
    } // 0.072
}
//*/
/*void Painter_2D::
clear( ::T_Rectangle rectangle )
  {
    int   i,
          left   = rectangle.left   (),
          right  = rectangle.right  (),
          top    = rectangle.top    (),
          bottom = rectangle.bottom (),
          width  = rectangle.width  (),
          height = rectangle.height ();

    if( left   < 0 || left > width_  )  MISTAKE;
    if( top    < 0 || top  > height_ )  MISTAKE;
    if( right  < 0 || right  >= width_  )  MISTAKE;
    if( bottom < 0 || bottom >= height_ )  MISTAKE;

    rgb   *pointer0,
          *pointer  = pixel_ + top * width_ + left,
          *end      = pixel_ + top * width_ + right;

    pointer0 = pointer;

    while( pointer < end )   *pointer++ = background_color_;
    for( i=1;  i<height;  i++ )
      {
        pointer = pixel_ + ( top + i ) * width_ + left;
        memcpy( pointer, pointer0, 4 * width );
      }
  }//*/
