/* 
    This file is part of the ChemCon molecular mechanics 
    and molecular graphics program.

    Copyright (C) 1997-2002  Alexei Nikitin 

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef PAINTER_2D_H
#define PAINTER_2D_H

#ifndef IMAGE_H
#include "Image.h"
#endif

/* //fix remove
Default Y direction is bottom to top.
If you need reversed direction then uncomment next line.
*/
//#define TOP_TO_BOTTOM_Y

/**
Painter is object that paint graphic primitives on the surface of Image.
*/
class Painter_2D
{
protected:
    typedef unsigned long  Rgb;
    typedef unsigned char  Byte;

    //Rgb                         * pixel_;
    Image                       &image_;
    int                         width_, height_;

    int                         X_, Y_;
    Rgb                         foreground_color_;
    Rgb                         background_color_;

    //      int                         pitch_;
    int                         corrected_height_;

public:
    Painter_2D( Image & image );
    ~Painter_2D(); //fix virtual

    Rgb                         rgb   ( Byte r, Byte g, Byte b);
    Byte                        alpha ( Rgb col );
    Byte                        red   ( Rgb col );
    Byte                        green ( Rgb col );
    Byte                        blue  ( Rgb col );

    void                        set_foreground_color( Rgb color );
    void                        set_foreground_color( Byte r, Byte g, Byte b );
    void                        set_background_color( Byte r, Byte g, Byte b );

    void                        point( int x, int y );
    void                        point( int x, int y, Rgb );
    void                        point( int x, int y, Byte r, Byte g, Byte b);
//  void                        get_pixel_colour( int x, int y, Byte *r, Byte *g, Byte *b );
    Rgb                         pixel_colour( int x, int y ); //fix name

    void                        square( int x, int y, int resolution );
    void                        square( int x, int y, 
                                        int red, int green, int blue, //fix to Byte or remove?
                                        int resolution );

    void                        move_to( int x, int y );

    void                        line_to( int x, int y );
    void                        Wu_line_to ( int x, int y );
    void                        Wu_line2_to( int x, int y );
    void                        Wu_line3_to( int x, int y );

    void                        line( int x1, int y1, int x2, int y2 );
    void                        Wu_line( int x1, int y1, int x2, int y2 );
    void                        Wu_line2( int x1, int y1, int x2, int y2 );
    void                        Wu_line3( int x1, int y1, int x2, int y2 );

    void                        clear();
//      void                      clear( ::T_Rectangle rectangle );
//    void                        clear( int left, int right,
//                                       int top,  int bottom );

    int                         width ();
    int                         height();

protected:
    unsigned short              us_abs( unsigned short x );

private:
    Painter_2D( const Painter_2D & );
    const Painter_2D & operator = ( const Painter_2D & );
};



inline Painter_2D::Rgb  Painter_2D::
rgb( Byte r, Byte g, Byte b)
{
    return (((Rgb)r)<<16)|(((Rgb)g)<<8)|((Rgb)b);
}

inline Painter_2D::Byte Painter_2D::alpha( Rgb col ){ return (Byte)((col)>>24)&0x00FFUL; }
inline Painter_2D::Byte Painter_2D::red  ( Rgb col ){ return (Byte)((col)>>16)&0x00FFUL; }
inline Painter_2D::Byte Painter_2D::green( Rgb col ){ return (Byte)((col)>>8 )&0x00FFUL; }
inline Painter_2D::Byte Painter_2D::blue ( Rgb col ){ return (Byte) (col)     &0x00FFUL; }

inline void Painter_2D::
set_foreground_color( Rgb color )
{
    foreground_color_ = color;
}

inline void Painter_2D::
set_foreground_color( Byte r, Byte g, Byte b )
{
    foreground_color_ = rgb(r,g,b);
}

inline void Painter_2D::
set_background_color( Byte r, Byte g, Byte b )
{
    background_color_= rgb(r,g,b);
}

inline void Painter_2D::
point( int x, int y )
{
    ((Rgb*)image_.scan_line( y ))[ x ] = foreground_color_;

/*    #ifdef TOP_TO_BOTTOM_Y
    *(pixel_ + ( corrected_height_ - y ) * width_ + x) = foreground_color_;
    #endif
    
    #ifndef TOP_TO_BOTTOM_Y
    *(pixel_ + y*width_ + x) = foreground_color_;
    #endif//*/
}

inline void Painter_2D::
point(int x, int y, Rgb color )
{
    ((Rgb*)image_.scan_line( y ))[ x ] = color;

/*    #ifdef TOP_TO_BOTTOM_Y
    *(pixel_ + ( corrected_height_ - y ) * width_ + x) = foreground_color_;
    #endif
    #ifndef TOP_TO_BOTTOM_Y
    *(pixel_ + y * width_ + x) = foreground_color_;
    #endif//*/
}

inline void Painter_2D::
point(int x, int y, Byte r, Byte g, Byte b)
{
    ((Rgb*)image_.scan_line( y ))[ x ]= rgb( r, g, b );

/*    #ifdef TOP_TO_BOTTOM_Y
    *(pixel_ + ( corrected_height_ - y ) * width_ + x) = rgb( r, g, b );
    #endif
    #ifndef TOP_TO_BOTTOM_Y
    *(pixel_ + y * width_ + x) = rgb( r, g, b );
    #endif//*/
}

inline Painter_2D::Rgb 
Painter_2D::pixel_colour( int x, int y )
  {
    return ((Rgb*)image_.scan_line( y ))[ x ];
    /*#ifdef TOP_TO_BOTTOM_Y
    return *(pixel_+ ( corrected_height_ - y ) * width_+ x);
    #endif
    #ifndef TOP_TO_BOTTOM_Y
    return *(pixel_+ y * width_+ x);
    #endif//*/
  }

inline void Painter_2D::
//square( int x, int y, int red, int green, int blue, int resolution )
square( int x, int y, int, int, int, int resolution )//fix
{
    set_foreground_color( Qt::red, Qt::green, Qt::blue ); //fix
    square( x, y, resolution );
}

inline void Painter_2D::
//square (int x, int y, int resolution)
square (int, int, int)
{
    throw;
}
/*
inline void Painter_2D::
square( int x, int y, int resolution )
{
    #ifdef TOP_TO_BOTTOM_Y
    Rgb *init_position = pixel_+ ( corrected_height_ - y ) * width_+ x;
    #endif
    #ifndef TOP_TO_BOTTOM_Y
    Rgb *init_position = pixel_+ y * width_+ x;
    #endif

    Rgb *pointer;

    switch( resolution )
    {
        case  1:
                  pointer    = init_position;
                  *pointer++ = foreground_color_;
                  *pointer++ = foreground_color_;

                  pointer    = init_position + width_;
                  *pointer++ = foreground_color_;
                  *pointer++ = foreground_color_;

                  break;

        case  2:
                  pointer    = init_position;
                  *pointer++ = foreground_color_;
                  *pointer++ = foreground_color_;
                  *pointer++ = foreground_color_;
                  *pointer++ = foreground_color_;

                  pointer    = init_position + width_;
                  *pointer++ = foreground_color_;
                  *pointer++ = foreground_color_;
                  *pointer++ = foreground_color_;
                  *pointer++ = foreground_color_;

                  pointer    = init_position + 2*width_;
                  *pointer++ = foreground_color_;
                  *pointer++ = foreground_color_;
                  *pointer++ = foreground_color_;
                  *pointer++ = foreground_color_;

                  pointer    = init_position + 3*width_;
                  *pointer++ = foreground_color_;
                  *pointer++ = foreground_color_;
                  *pointer++ = foreground_color_;
                  *pointer++ = foreground_color_;
                  break;

        case  3:                                //fix 4?
                  pointer    = init_position;
                  *pointer++ = foreground_color_;
                  *pointer++ = foreground_color_;
                  *pointer++ = foreground_color_;
                  *pointer++ = foreground_color_;
                  *pointer++ = foreground_color_;
                  *pointer++ = foreground_color_;
                  *pointer++ = foreground_color_;
                  *pointer++ = foreground_color_;

                  pointer    = init_position + width_;
                  *pointer++ = foreground_color_;
                  *pointer++ = foreground_color_;
                  *pointer++ = foreground_color_;
                  *pointer++ = foreground_color_;
                  *pointer++ = foreground_color_;
                  *pointer++ = foreground_color_;
                  *pointer++ = foreground_color_;
                  *pointer++ = foreground_color_;

                  pointer    = init_position + 2*width_;
                  *pointer++ = foreground_color_;
                  *pointer++ = foreground_color_;
                  *pointer++ = foreground_color_;
                  *pointer++ = foreground_color_;
                  *pointer++ = foreground_color_;
                  *pointer++ = foreground_color_;
                  *pointer++ = foreground_color_;
                  *pointer++ = foreground_color_;

                  pointer    = init_position + 3*width_;
                  *pointer++ = foreground_color_;
                  *pointer++ = foreground_color_;
                  *pointer++ = foreground_color_;
                  *pointer++ = foreground_color_;
                  *pointer++ = foreground_color_;
                  *pointer++ = foreground_color_;
                  *pointer++ = foreground_color_;
                  *pointer++ = foreground_color_;

                  pointer    = init_position + 4*width_;
                  *pointer++ = foreground_color_;
                  *pointer++ = foreground_color_;
                  *pointer++ = foreground_color_;
                  *pointer++ = foreground_color_;
                  *pointer++ = foreground_color_;
                  *pointer++ = foreground_color_;
                  *pointer++ = foreground_color_;
                  *pointer++ = foreground_color_;

                  pointer    = init_position + 5*width_;
                  *pointer++ = foreground_color_;
                  *pointer++ = foreground_color_;
                  *pointer++ = foreground_color_;
                  *pointer++ = foreground_color_;
                  *pointer++ = foreground_color_;
                  *pointer++ = foreground_color_;
                  *pointer++ = foreground_color_;
                  *pointer++ = foreground_color_;

                  pointer    = init_position + 6*width_;
                  *pointer++ = foreground_color_;
                  *pointer++ = foreground_color_;
                  *pointer++ = foreground_color_;
                  *pointer++ = foreground_color_;
                  *pointer++ = foreground_color_;
                  *pointer++ = foreground_color_;
                  *pointer++ = foreground_color_;
                  *pointer++ = foreground_color_;

                  pointer    = init_position + 7*width_;
                  *pointer++ = foreground_color_;
                  *pointer++ = foreground_color_;
                  *pointer++ = foreground_color_;
                  *pointer++ = foreground_color_;
                  *pointer++ = foreground_color_;
                  *pointer++ = foreground_color_;
                  *pointer++ = foreground_color_;
                  *pointer++ = foreground_color_;

                  break;

        case  0:  point( x, y );
                  break;

        default:  throw;
    }
}
//*/

inline void Painter_2D::move_to    ( int x, int y ) { X_ = x; Y_ = y; }

inline void Painter_2D::line_to    ( int x, int y ) { line    ( X_,Y_, x,y ); }
inline void Painter_2D::Wu_line_to ( int x, int y ) { Wu_line ( X_,Y_, x,y ); }
inline void Painter_2D::Wu_line2_to( int x, int y ) { Wu_line2( X_,Y_, x,y ); }
inline void Painter_2D::Wu_line3_to( int x, int y ) { Wu_line3( X_,Y_, x,y ); }
//fix reverse Wu_line_to and Wu_line



inline int Painter_2D::width () { return width_;  }
inline int Painter_2D::height() { return height_; }

inline unsigned short Painter_2D::
us_abs( unsigned short x )
{
//    if( x & 0x8000 )    return (x ^ 0xFFFF) + 1;
//    else                return x;
    unsigned short condition = (x & 0x8000)>>15;
    return ((x ^ 0xFFFF) + 1)*condition + x*(1-condition);
}


#endif //PAINTER_2D_H

