#include "OpenRM_model_view.h"

#include <QWorkspace>//fix

#include "Log.h"

#include "Model_view.h"
#include "Event.h"
#include "Model.h"
#include "Model_kit.h"
#include "Project.h"
#include "Atom.h"
#include "Atom_kit.h"
#include "QRMWidget.h"
#include "Form_model.h"
#include "Form_main_window.h"
#include "Qt_application.h"

#include <rm/rm.h>

namespace MM
{

OpenRM_model_view::
OpenRM_model_view (Model      & model,
                   QRMWidget  & rm_widget,
                   Form_model & MDI_window)
:
    model_      (&model),
    rm_widget_  (&rm_widget),
    MDI_window_ (&MDI_window)
{
    rm_widget.set_model_view (this);
}

Model_view * OpenRM_model_view::
clone (Model &model, const char * name) const       //fix  use it?
{
    Form_main_window & main_window = Qt_application::singleton().main_window();
//3    Form_model *mdi_window = new Form_model (main_window);

    QRMWidget *rm_widget = new QRMWidget;
    Form_model *mdi_window = new Form_model (model, rm_widget);
    mdi_window->setCaption (name);


    /*Form_model *mdi_window = new Form_model;
    mdi_window->setCaption (name);
    QRMWidget *rm_widget = new QRMWidget (mdi_window);
    mdi_window->set_central_widget (rm_widget);//*/
    mdi_window->focus_in.subscribe (model.kit().view(), &Model_view::set_focus);
    main_window.add_MDI_window (mdi_window, name);

    OpenRM_model_view *twin = 
        new OpenRM_model_view (model, *rm_widget, *mdi_window);
    model.kit().add_view (twin);

    model.atom_prototype().kit().add_visual
        (new OpenRM_atom (rm_widget->object_node(), rm_widget));

    return twin;
}

void OpenRM_model_view::
fit_to_screen()
{
    if (model_->atom_count() == 0)
        return;

    rm_widget_->fit_to_screen();
}

void OpenRM_model_view::
flush_camera ()
{
    rm_widget_->flush_camera();
}

void OpenRM_model_view::
set_level_of_details (int lod)
{
    //fix to Group_of_visual_atoms
    rmPipeSetDisplayListEnable (rm_widget_->pipe(), RM_FALSE);

    for (int i=0;  i<model_->atom_count();  ++i)
    {
//        model_.atom(i).kit().openRM().set_level_of_details (lod);
        model_->atom(i).kit().visual().set_level_of_details (lod);
    }

    //rmFrame (rm_widget_.pipe(), rm_widget_.root_node());
    //rmPipeSetDisplayListEnable (rm_widget_.pipe(), RM_TRUE);
}

void OpenRM_model_view::
set_focus ()
{
    Project::singleton().set_current_model (*model_);

//    QWidget * parent = (QWidget *)rm_widget_->parent();
    //parent->raise ();
    Qt_application::singleton ().main_window().workspace->
        setActiveWindow (MDI_window_);
}

void OpenRM_model_view::
get_dimensions (double &center_x,
                double &center_y,
                double &center_z,
                double &scale)
{
    double atoms = model_->atom_count();
    if (atoms == 0)
    {
        center_x = 0.;
        center_y = 0.;
        center_z = 0.;
        scale    = 0.;
        return;
    }

    const Atom & first_atom = model_->atom(0);

    double sum_x = 0, x_min = first_atom.x(), x_max = first_atom.x();
    double sum_y = 0, y_min = first_atom.y(), y_max = first_atom.y();
    double sum_z = 0, z_min = first_atom.z(), z_max = first_atom.z();

    for (int i=0;  i<atoms;  ++i)
    {
        const Atom & current_atom = model_->atom (i);
        double current_x = current_atom. x();
        double current_y = current_atom. y();
        double current_z = current_atom. z();

        sum_x += current_x;
        sum_y += current_y;
        sum_z += current_z;

        x_min = current_x < x_min ? current_x : x_min;
        x_max = current_x > x_max ? current_x : x_max;
        y_min = current_y < y_min ? current_y : y_min;
        y_max = current_y > y_max ? current_y : y_max;
        z_min = current_z < z_min ? current_z : z_min;
        z_max = current_z > z_max ? current_z : z_max;
    }

    center_x = sum_x / atoms;
    center_y = sum_y / atoms;
    center_z = sum_z / atoms;

    double dx = x_max - x_min;
    double dy = y_max - y_min;
    double dz = z_max - z_min;

    scale = sqrt (dx*dx + dy*dy + dz*dz) + 5.;
}

}//MM



