#include "Qt_table_table_presentation.h"

#include "Table.h"

#include <q3table.h>
#include <q3header.h> 

namespace MM
{

Qt_table_table_presentation::
Qt_table_table_presentation (Q3Table & table)
:
    qt_table_(&table)
{
}

Qt_table_table_presentation::
Qt_table_table_presentation (Q3Table * table)
:
    qt_table_(table),
    adoptee_ (table)
{
}

Qt_table_table_presentation::
Qt_table_table_presentation ()
:
    qt_table_(new Q3Table()),
    adoptee_ (qt_table_)
{
}

Table_presentation * Qt_table_table_presentation::
clone () const
{
    return new Qt_table_table_presentation (*qt_table_);
}

void Qt_table_table_presentation::
show ()
{
    full_update ();
}

void Qt_table_table_presentation::
full_update ()
{
    INVARIANT;

    int row, column;
    int height = table().height();
    int width  = table().width ();
    qt_table_->setNumRows(height);
    qt_table_->setNumCols(width);

    if (table().has_column_labels())
    {
        Q3Header *header = qt_table_->horizontalHeader (); 

        for (column = 0;  column < width;  ++column)
        {
            Text text = table().column_label (column);
            header->setLabel (column, QString::fromLocal8Bit (text.c_str()));
        }
    }

    if (table().has_row_labels())
    {
        Q3Header *header = qt_table_->verticalHeader (); 

        for (row = 0;  row < height;  ++row)
        {
            Text text = table().row_label (row);
            header->setLabel (row, QString::fromLocal8Bit (text.c_str()));
        }
    }

    for (row=0;  row<height;  ++row)
    {
        for (column=0;  column<width;  ++column)
        {
            Text text;
            table().get_text (row, column, text);
            qt_table_->setText(row, column, QString::fromLocal8Bit (text.c_str()));
        }
    }
}

}//MM
